package pps.core.common.cache;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.util.CollectionUtils;
import pps.core.common.entity.SysTradeInterfaceWhiteEnt;
import pps.core.common.mapper.SysTradeInterfaceWhiteMapper;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.cache.XCacheLife;
import xstartup.cache.XCacheObject;
import xstartup.cache.XListCache;

import java.util.List;

/**
 * 交易平台用户权限缓存
 * 优先判断 用户权限，为空时判断角色权限
 */
public class TradeUserPermissionCache implements XCacheObject, XCacheLife {
    private String serviceName;
    private String methodName;

    public TradeUserPermissionCache() {
    }

    public TradeUserPermissionCache(String serviceName, String methodName) {
        this.serviceName = serviceName;
        this.methodName = methodName;
    }

    /**
     * 检查缓存是否存在指定的code
     *
     * @param context
     * @param roleId
     * @param serviceName
     * @param methodName
     * @return
     */
    public static boolean exist(XContext context, Long roleId, String serviceName, String methodName) {
        return Tool.get(Tool.class).exist(context, roleId.toString(), serviceName + "#" + methodName, TradeUserPermissionCache.class);
    }

    /**
     * 从缓存中读取
     *
     * @param context
     * @param roleId
     * @return
     */
    public static List<TradeUserPermissionCache> list(XContext context, Long roleId) {
        return Tool.get(Tool.class).list(context, roleId.toString(), TradeUserPermissionCache.class);
    }

    /**
     * 设置缓存
     *
     * @param context
     * @param roleId
     * @param caches
     */
    public static void set(XContext context, Long roleId, List<TradeUserPermissionCache> caches) {
        Tool.get(Tool.class).set(context, roleId.toString(), caches);
    }

    /**
     * 删除缓存
     *
     * @param context
     * @param roleId
     */
    public static void delete(XContext context, Long roleId) {
        Tool.get(Tool.class).delete(context, roleId.toString(), TradeUserPermissionCache.class);
    }

    public String getServiceName() {
        return serviceName;
    }

    public void setServiceName(String serviceName) {
        this.serviceName = serviceName;
    }

    public String getMethodName() {
        return methodName;
    }

    public void setMethodName(String methodName) {
        this.methodName = methodName;
    }

    @Override
    public String getCacheKey() {
        return this.serviceName + "#" + this.methodName;
    }

    @Override
    public Integer getDuration() {
        //过期时间（秒）
        // 1天
//        return 24 * 3600;
        // 5分钟
        return 5 * 60;
    }

    static class Tool extends XListCache<TradeUserPermissionCache> {
        @Override
        protected List<TradeUserPermissionCache> restore(XContext context, String listKey) {
            SysTradeInterfaceWhiteMapper whiteMapper = context.getBean(SysTradeInterfaceWhiteMapper.class);
            QueryWrapper<SysTradeInterfaceWhiteEnt> whiteEntQueryWrapper = new QueryWrapper<>();
            List<SysTradeInterfaceWhiteEnt> whiteEntList = whiteMapper.selectList(whiteEntQueryWrapper);
            if (!CollectionUtils.isEmpty(whiteEntList)) {
                return XCopyUtils.copyNewList(whiteEntList, TradeUserPermissionCache.class);
            }
            return null;
        }
    }
}
