package pps.core.common.job;

import com.google.common.base.Strings;
import pps.core.common.utils.*;
import xstartup.base.XStartup;
import xstartup.base.util.XJsonUtils;

import java.util.Set;
import java.util.concurrent.atomic.AtomicBoolean;

public class CounterJob5S {

    public static final String CLUSTER_GLOBAL_COUNTER = "counter.cluster_global_counter";
    public static final String CLUSTER_GLOBAL_COUNTER_LOCK = "counter.cluster_global_counter_lock";
    public static final TimeSpanChecker publishChecker = new TimeSpanChecker(15, true);
    public static boolean EnableJob = false;
    //    public static final TimeSpanChecker onlineChecker = new TimeSpanChecker(60, true);
    private static AtomicBoolean master = new AtomicBoolean(false);
//    private static HashMap<String, String> onlineNodes = new HashMap<String, String>();

    public static void executeJob() {
        if (!EnableJob) {
            return;
        }
        if (!publishChecker.checkNow()) {
            return;
        }
        CounterBuilder.globalCounterBuilder.addCounter("CounterJob-Timer");
        CounterBuilder.globalCounterBuilder.setFieldValue("CounterJob-Timer-LatestTime", DateUtil.getCurrentDateTime());
        CounterBuilder.globalCounterBuilder.setFieldValue("__IPS__", IpUtils.getHostIPs(true));
        CounterBuilder.globalCounterBuilder.setFieldValue("__TASKS_RUNNING__", GlobalExecutorService.INSTANCE.getRunningTasksCount() + "");
        CounterBuilder.globalCounterBuilder.setFieldValue("__TASKS_DONE__", GlobalExecutorService.INSTANCE.getTasksCount() + "");
        CounterBuilder.globalCounterBuilder.setFieldValue("__TASKS_RUNNING(SCADA)___", ScadaExecutorService.INSTANCE.getRunningTasksCount() + "");
        CounterBuilder.globalCounterBuilder.setFieldValue("__TASKS_DONE(SCADA)__", ScadaExecutorService.INSTANCE.getTasksCount() + "");
        CounterBuilder.globalCounterBuilder.setFieldValue("__TRACEBUILDERS__", TraceHelper.getTraceBuildersCount() + "");

        DynObj dynObj = CounterBuilder.globalCounterBuilder.toDynObj();
        dynObj.put(CounterBuilder.COUNTER_NAME_INSTANCE, CounterBuilder.globalCounterBuilder.counterId);
        dynObj.put("__LATEST_TIMER_TIME__", DateUtil.getCurrentDateTime());
        dynObj.put("__PUBLISH_TIME__", DateUtil.getCurrentDateTime());
        dynObj.put("__THREADS__", ThreadHelper.buildThreadsSimpleInfos(null));
        long publishStart = System.currentTimeMillis();
        publish(dynObj);
        long publishEnd = System.currentTimeMillis();
        CounterBuilder.globalCounterBuilder.setFieldValue("CounterJob-Publish-UsedSecnods", (publishEnd - publishStart) / 1000 + "");
//
//        if (onlineChecker.checkNow()) {
//            checkOnLine();
//        }
    }

    public static String getAppClusterName() {
        String appName = XStartup.getApplication().getName();
        String env = XStartup.getApplication().getEnv().name();
        String appClusterName = CLUSTER_GLOBAL_COUNTER + "_" + env + "_" + appName;
        return appClusterName;
    }

    public static String getAppClusterLockName() {
        String appName = XStartup.getApplication().getName();
        String env = XStartup.getApplication().getEnv().name();
        String appClusterLockName = CLUSTER_GLOBAL_COUNTER_LOCK + "_" + env + "_" + appName;
        return appClusterLockName;
    }

    private static void publish(DynObj dynObj) {
        CounterBuilder.globalCounterBuilder.addCounter("CounterJob-publish-begin");
        try {
            String myNodeKey = String.format("counter.%s.%s", dynObj.getString(CounterBuilder.DEPLOY_KEY, "UnknownDeploy"), CounterBuilder.globalCounterBuilder.counterId);
            dynObj.put(CounterBuilder.COUNTER_NAME_OWNER, myNodeKey);
            String cacheType = XStartup.getCurrent().getProperty("x.cache.type");
            String appClusterName = getAppClusterName();
            String appClusterLockName = getAppClusterLockName();

            if ("redis-cluster".equals(cacheType)) {

                JedisHelper.consumeDefalutJedisCluster((jedis) -> {
                    // 记录到集群中
                    jedis.sadd(appClusterName, myNodeKey);
                    // 集群3分钟过期
                    jedis.expire(appClusterName, 3 * 20);

                    boolean isMaster = false;
                    long success = jedis.setnx(appClusterLockName, myNodeKey);
                    if (success != 0) {
                        jedis.expire(appClusterLockName, 40);
                        isMaster = true;
                    }
                    String lockKey = jedis.get(appClusterLockName);
                    if (myNodeKey.equals(lockKey)) {
                        jedis.expire(appClusterLockName, 40);
                        isMaster = true;
                    }
                    master.set(isMaster);

                    CounterBuilder.globalCounterBuilder.setFieldValue("__MASTER__", isMaster + "");
                    if (isMaster) {
                        if (Strings.isNullOrEmpty(CounterBuilder.globalCounterBuilder.getFieldValue("__MASTER-TIME__"))) {
                            CounterBuilder.globalCounterBuilder.setFieldValue("__MASTER-TIME__", DateUtil.getCurrentDateTime());
                            CounterBuilder.globalCounterBuilder.addCounter("__MASTER-TIMES__");
                        }
                    } else {
                        if (!Strings.isNullOrEmpty(CounterBuilder.globalCounterBuilder.getFieldValue("__MASTER-TIME__"))) {
                            CounterBuilder.globalCounterBuilder.setFieldValue("__MASTER-TIME__", "");
                        }
//                        onlineNodes.clear();
                    }
                    CounterBuilder.globalCounterBuilder.addCounter("CounterJob-PublishToRedis");
                    CounterBuilder.globalCounterBuilder.setFieldValue("CounterJob-PublishToRedis-TIME", DateUtil.getCurrentDateTime());
                    // 记录到redis ,3 * 30s过期
                    jedis.setex(myNodeKey, 3 * 20, XJsonUtils.toJson(dynObj));

                    if (isMaster) {
                        CounterBuilder.globalCounterBuilder.addCounter("CounterJob-Timer-MASTER");
                        Set<String> nodeKeys = jedis.smembers(appClusterName);
                        nodeKeys.forEach(k -> {
                            if (!jedis.exists(k)) {
                                jedis.srem(appClusterName, k);
                                CounterBuilder.globalCounterBuilder.addCounter("Remove-Node");
//                                if (onlineNodes.containsKey(k)) {
//                                    onlineNodes.remove(k);
//                                }
                            } else {
//                                if (!onlineNodes.containsKey(k) && !k.equals(myNodeKey)) {
//                                    onlineNodes.put(k, "");
//                                }
                            }
                        });
                    } else {
                        CounterBuilder.globalCounterBuilder.addCounter("CounterJob-Timer-NOTMASTER");
                    }
                });
            } else {
                JedisHelper.consumeDefalutJedis((jedis) -> {
                    // 记录到redis ,3 * 30s过期
//                    jedis.setex(nodeKey, 3 * 20, XJsonUtils.toJson(dynObj));
//                    CounterBuilder.globalCounterBuilder.addCounter("CounterJob-PublishToRedis");
//                    CounterBuilder.globalCounterBuilder.setFieldValue("CounterJob-PublishToRedis-TIME", DateUtil.getCurrentDateTime());

                    // 记录到集群中
                    jedis.sadd(appClusterName, myNodeKey);
                    // 集群3分钟过期
                    jedis.expire(appClusterName, 3 * 20);

                    boolean isMaster = false;
                    long success = jedis.setnx(appClusterLockName, myNodeKey);
                    if (success != 0) {
                        jedis.expire(appClusterLockName, 40);
                        isMaster = true;
                    }
                    String lockKey = jedis.get(appClusterLockName);
                    if (myNodeKey.equals(lockKey)) {
                        jedis.expire(appClusterLockName, 40);
                        isMaster = true;
                    }
                    master.set(isMaster);

                    CounterBuilder.globalCounterBuilder.setFieldValue("__MASTER__", isMaster + "");
                    if (isMaster) {
                        if (Strings.isNullOrEmpty(CounterBuilder.globalCounterBuilder.getFieldValue("__MASTER-TIME__"))) {
                            CounterBuilder.globalCounterBuilder.setFieldValue("__MASTER-TIME__", DateUtil.getCurrentDateTime());
                            CounterBuilder.globalCounterBuilder.addCounter("__MASTER-TIMES__");
                        }
                    } else {
                        if (!Strings.isNullOrEmpty(CounterBuilder.globalCounterBuilder.getFieldValue("__MASTER-TIME__"))) {
                            CounterBuilder.globalCounterBuilder.setFieldValue("__MASTER-TIME__", "");
                        }
//                        onlineNodes.clear();
                    }
                    CounterBuilder.globalCounterBuilder.addCounter("CounterJob-PublishToRedis");
                    CounterBuilder.globalCounterBuilder.setFieldValue("CounterJob-PublishToRedis-TIME", DateUtil.getCurrentDateTime());
                    // 记录到redis ,3 * 30s过期
                    jedis.setex(myNodeKey, 3 * 20, XJsonUtils.toJson(dynObj));

                    if (isMaster) {
                        CounterBuilder.globalCounterBuilder.addCounter("CounterJob-Timer-MASTER");
                        Set<String> nodeKeys = jedis.smembers(appClusterName);
                        nodeKeys.forEach(k -> {
                            if (!jedis.exists(k)) {
                                jedis.srem(appClusterName, k);
                                CounterBuilder.globalCounterBuilder.addCounter("Remove-Node");
//                                if (onlineNodes.containsKey(k)) {
//                                    onlineNodes.remove(k);
//                                }
                            } else {
//                                if (!onlineNodes.containsKey(k) && !k.equals(myNodeKey)) {
//                                    onlineNodes.put(k, "");
//                                }
                            }
                        });
                    } else {
                        CounterBuilder.globalCounterBuilder.addCounter("CounterJob-Timer-NOTMASTER");
                    }
                });
            }
        } catch (Exception ex) {
            CounterBuilder.globalCounterBuilder.addCounter(ex, CounterJob5S.class);
//            CounterBuilder.globalCounterBuilder.addCounter("Exception-" + CounterJob.class.getSimpleName() + "-" + ex.getClass().getSimpleName());
        }
    }
//
//    public static void checkOnLine() {
//        CounterBuilder.globalCounterBuilder.addCounter("CounterJob-checkOnLine-begin");
//        if (!master.get()) {
//            return;
//        }
//        try {
//            String cacheType = XStartup.getCurrent().getProperty("x.cache.type");
//
//            if ("redis-cluster".equals(cacheType)) {
//
//                JedisHelper.consumeDefalutJedisCluster((jedis) -> {
//                    onlineNodes.keySet().stream().collect(Collectors.toList()).forEach(nodeKey -> {
//                        String v = onlineNodes.get(nodeKey);
//                        if (Strings.isNullOrEmpty(v)) {
//                            String jedisValue = jedis.get(nodeKey);
//                            if (!Strings.isNullOrEmpty(jedisValue)) {
//                                onlineNodes.put(nodeKey, jedisValue);
//                            } else {
//                                onlineNodes.remove(nodeKey);
//                            }
//                        } else {
//                            String jedisValue = jedis.get(nodeKey);
//                            if (jedisValue.equals(v)) {
//                                onlineNodes.remove(nodeKey);
//                                jedis.del(nodeKey);
//                                CounterBuilder.globalCounterBuilder.addCounter("CounterJob-checkOnLine-removed");
//                            } else {
//                                onlineNodes.put(nodeKey, jedisValue);
//                            }
//                        }
//
//                    });
//                });
//            } else {
//                JedisHelper.consumeDefalutJedis((jedis) -> {
//                    onlineNodes.keySet().stream().collect(Collectors.toList()).forEach(nodeKey -> {
//                        String v = onlineNodes.get(nodeKey);
//                        if (Strings.isNullOrEmpty(v)) {
//                            String jedisValue = jedis.get(nodeKey);
//                            if (!Strings.isNullOrEmpty(jedisValue)) {
//                                onlineNodes.put(nodeKey, jedisValue);
//                            } else {
//                                onlineNodes.remove(nodeKey);
//                            }
//                        } else {
//                            String jedisValue = jedis.get(nodeKey);
//                            if (jedisValue.equals(v)) {
//                                onlineNodes.remove(nodeKey);
//                                jedis.del(nodeKey);
//                                CounterBuilder.globalCounterBuilder.addCounter("CounterJob-checkOnLine-removed");
//                            } else {
//                                onlineNodes.put(nodeKey, jedisValue);
//                            }
//                        }
//
//                    });
//                });
//            }
//        } catch (Exception ex) {
//            CounterBuilder.globalCounterBuilder.addCounter(ex, CounterJob5S.class);
//        }
//    }

}
