package pps.core.common.job;

import pps.core.common.utils.CounterBuilder;
import pps.core.common.utils.TimeSpanChecker;
import pps.core.common.utils.TraceHelper;
import xstartup.base.XContext;
import xstartup.base.tool.XStorageTool;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Pattern;


public class TempCleanJob5S {

    public static final TimeSpanChecker jobChecker = new TimeSpanChecker(60 * 60, true);
    public static final ConcurrentHashMap<String, String[]> tempFilePatttern = new ConcurrentHashMap<String, String[]>();
    public static boolean EnableJob = true;

    static {
        tempFilePatttern.put("/var/tmp/", new String[]{"^tracelog-\\d+.txt$"});
    }

    public static void executeTimerJob(XContext context) {
        if (!EnableJob) {
            return;
        }
        if (!jobChecker.checkNow()) {
            return;
        }
        executeJob(context);
    }

    public static void executeJob(XContext context) {
        tempFilePatttern.forEach((k, v) -> {
            checkAndCleanTempFile(context, k, v);
        });
    }

    private static void checkAndCleanTempFile(XContext context, String folder, String[] filePatterns) {
        try {
            String absoluteFolderPath = XStorageTool.getAbsolutePath(folder);
            TraceHelper.appendTraceLog(context, "absolutePath", absoluteFolderPath);
            File absoluteFolderPathFile = new File(absoluteFolderPath);
            if (!absoluteFolderPathFile.exists()) {
                TraceHelper.appendTraceLog(context, "ERROR", absoluteFolderPath + " not exists");
                return;
            }

            String[] files = absoluteFolderPathFile.list();
            for (String file : files) {
                TraceHelper.appendTraceLog(context, "", file);
                for (String pattern : filePatterns) {
                    if (Pattern.compile(pattern).matcher(file).find()) {
                        TraceHelper.appendTraceLog(context, "FOUND", file);
                        File f = new File(absoluteFolderPath, file);
                        try {
                            if (checkTimeAndDeleteFile(f)) {
                                TraceHelper.appendTraceLog(context, "Removed", absoluteFolderPathFile.getAbsolutePath());
                            }
                        } catch (Exception ex2) {
                            TraceHelper.appendTraceLog(context, ex2);
                        }
                        break;
                    }
                }
            }

        } catch (Exception ex) {
            CounterBuilder.globalCounterBuilder.addCounter(ex);
            CounterBuilder.globalCounterBuilder.addCounter("checkAndCleanTempFile-ERROR-" + folder);
        }
    }

    private static boolean checkTimeAndDeleteFile(File file) throws IOException {

        Path path = file.toPath();

        // creating BasicFileAttributes class object using
        // readAttributes method
        BasicFileAttributes file_att = Files.readAttributes(
                path, BasicFileAttributes.class);

        // printing the file creation time by calling
        // creationTime() method
//        System.out.printf("File Creation Time %s%n ",
//                file_att.creationTime());
        long createTime = file_att.creationTime().toMillis();
        long now = System.currentTimeMillis();
        if (now - createTime < TimeSpanChecker.TIMESPAN_MS_1d) {
            return false;
        }

        file.delete();
        return true;
    }


}
