package pps.core.common.provider.impl;

import pps.core.common.config.MqConfig;
import pps.core.common.mq.AccessLogMqMessage;
import xstartup.annotation.XImplement;
import xstartup.base.XContext;
import xstartup.base.mq.XMqFactory;
import xstartup.base.mq.XMqProduceQueue;
import xstartup.base.mq.XMqProducer;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XServiceResult;
import xstartup.feature.api.provider.data.XSaveApiAccessLogParam;
import xstartup.feature.api.provider.impl.async.XAsyncApiAccessLogProvider;
import xstartup.feature.api.provider.impl.async.XAsyncApiAccessLogSaveThread;

@XImplement
public class PpsApiAccessLogProviderImpl extends XAsyncApiAccessLogProvider {
    @Override
    protected <T extends XAsyncApiAccessLogSaveThread> Class<T> getThreadClass() {
        return (Class<T>) SaveThread.class;
    }

    public static class SaveThread extends XAsyncApiAccessLogSaveThread {
        @Override
        protected XServiceResult saveBatch(XContext context, Batch batch) {
            final String logTopic = MqConfig.accessLogMq.getValue(context);
            if (logTopic == null) {
                context.getLogger().warn("未配置访问日志topic");
                return XServiceResult.OK;
            }

            for (XSaveApiAccessLogParam item : batch.getItems()) {
                context.log().info("save access log:{}", item);
                //todo:记录接口访问日志
                XMqProduceQueue queue = new XMqProduceQueue.Builder()
                        .queue(logTopic)
                        .build();
                XMqProducer<AccessLogMqMessage> producer = XMqFactory.createProducer(context, queue, AccessLogMqMessage.class);

                AccessLogMqMessage message = new AccessLogMqMessage();
                XCopyUtils.copyObject(item, message);
                message.setUserId(item.getUser().getCode());
                message.setResponseValue(message.getResponseValue());
                producer.publish(context, message).logIfFail(context);
            }
            return XServiceResult.OK;
        }
    }
}
