package pps.core.common.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.common.entity.BaseSysConfigEnt;
import pps.core.common.entity.BaseSysConfigView;
import pps.core.common.mapper.BaseSysConfigMapper;
import pps.core.common.mapper.BaseSysConfigViewMapper;
import pps.core.common.service.data.base_sys_config.*;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;

@XService
public class BaseSysConfigService {
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBaseSysConfig(XContext context, CreateBaseSysConfigInput input) {
        BaseSysConfigMapper mapper = context.getBean(BaseSysConfigMapper.class);
        BaseSysConfigEnt entity = new BaseSysConfigEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBaseSysConfig(XContext context, UpdateBaseSysConfigInput input) {
        BaseSysConfigMapper mapper = context.getBean(BaseSysConfigMapper.class);
        QueryWrapper<BaseSysConfigEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseSysConfigEnt::getId, input.getId());
        BaseSysConfigEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBaseSysConfig(XContext context, DeleteBaseSysConfigInput input) {
        BaseSysConfigMapper mapper = context.getBean(BaseSysConfigMapper.class);
        QueryWrapper<BaseSysConfigEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseSysConfigEnt::getId, input.getId());
        BaseSysConfigEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBaseSysConfigOutput> getBaseSysConfig(XContext context, GetBaseSysConfigInput input) {
        BaseSysConfigMapper mapper = context.getBean(BaseSysConfigMapper.class);
        QueryWrapper<BaseSysConfigEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseSysConfigEnt::getId, input.getId());
        BaseSysConfigEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseSysConfigOutput output = new GetBaseSysConfigOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseSysConfigOutput> queryBaseSysConfig(XContext context, QueryBaseSysConfigInput input) {
        BaseSysConfigMapper mapper = context.getBean(BaseSysConfigMapper.class);
        QueryWrapper<BaseSysConfigEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(BaseSysConfigEnt::getId, input.getId())
                .like(BaseSysConfigEnt::getName, input.getName())
                .like(BaseSysConfigEnt::getMainKeys, input.getKey())
                .like(BaseSysConfigEnt::getValue, input.getValue())
                .like(BaseSysConfigEnt::getCreateById, input.getCreateById())
                .like(BaseSysConfigEnt::getCreateByName, input.getCreateByName())
                .like(BaseSysConfigEnt::getModifyById, input.getModifyById())
                .like(BaseSysConfigEnt::getModifyByName, input.getModifyByName());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseSysConfigOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseSysConfigOutput> dynamicQueryBaseSysConfig(XContext context, CustomQueryInput input) {
        BaseSysConfigMapper mapper = context.getBean(BaseSysConfigMapper.class);
        return XMapperHelper.query(mapper, input, BaseSysConfigEnt.class, QueryBaseSysConfigOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBaseSysConfigViewOutput> getBaseSysConfigView(XContext context, GetBaseSysConfigViewInput input) {
        BaseSysConfigViewMapper mapper = context.getBean(BaseSysConfigViewMapper.class);
        BaseSysConfigView record = new BaseSysConfigView();
        XCopyUtils.copyObject(input, record);
        BaseSysConfigView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseSysConfigViewOutput output = new GetBaseSysConfigViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseSysConfigViewOutput> queryBaseSysConfigView(XContext context, QueryBaseSysConfigViewInput input) {
        BaseSysConfigViewMapper mapper = context.getBean(BaseSysConfigViewMapper.class);
        BaseSysConfigView record = new BaseSysConfigView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BaseSysConfigView> list = mapper.selectList(record);
        PageInfo<BaseSysConfigView> pageInfo = new PageInfo<>(list);
        List<QueryBaseSysConfigViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseSysConfigViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
