package pps.core.common.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.common.entity.BaseSysLogEnt;
import pps.core.common.entity.BaseSysLogView;
import pps.core.common.mapper.BaseSysLogMapper;
import pps.core.common.mapper.BaseSysLogViewMapper;
import pps.core.common.service.data.base_sys_log.*;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;

@XService
public class BaseSysLogService {
    @XApiPost
    public XServiceResult createBaseSysLog(XContext context, CreateBaseSysLogInput input) {
        BaseSysLogMapper mapper = context.getBean(BaseSysLogMapper.class);
        BaseSysLogEnt entity = new BaseSysLogEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XApiPost
    public XServiceResult updateBaseSysLog(XContext context, UpdateBaseSysLogInput input) {
        BaseSysLogMapper mapper = context.getBean(BaseSysLogMapper.class);
        QueryWrapper<BaseSysLogEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseSysLogEnt::getId, input.getId());
        BaseSysLogEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiPost
    public XServiceResult deleteBaseSysLog(XContext context, DeleteBaseSysLogInput input) {
        BaseSysLogMapper mapper = context.getBean(BaseSysLogMapper.class);
        QueryWrapper<BaseSysLogEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseSysLogEnt::getId, input.getId());
        BaseSysLogEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiGet
    public XSingleResult<GetBaseSysLogOutput> getBaseSysLog(XContext context, GetBaseSysLogInput input) {
        BaseSysLogMapper mapper = context.getBean(BaseSysLogMapper.class);
        QueryWrapper<BaseSysLogEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseSysLogEnt::getId, input.getId());
        BaseSysLogEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseSysLogOutput output = new GetBaseSysLogOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiGet
    public XPageResult<QueryBaseSysLogOutput> queryBaseSysLog(XContext context, QueryBaseSysLogInput input) {
        BaseSysLogMapper mapper = context.getBean(BaseSysLogMapper.class);
        QueryWrapper<BaseSysLogEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseSysLogEnt::getId, input.getId())
                .like(BaseSysLogEnt::getSystemName, input.getSystemName())
                .like(BaseSysLogEnt::getScopeName, input.getScopeName())
                .like(BaseSysLogEnt::getModuleName, input.getModuleName())
                .like(BaseSysLogEnt::getServiceName, input.getServiceName())
                .like(BaseSysLogEnt::getActionName, input.getActionName())
                .like(BaseSysLogEnt::getRequestMethod, input.getRequestMethod())
                .like(BaseSysLogEnt::getRequestIp, input.getRequestIp())
                .like(BaseSysLogEnt::getBrowserInfo, input.getBrowserInfo())
                .like(BaseSysLogEnt::getSource, input.getSource())
                .like(BaseSysLogEnt::getRequestValue, input.getRequestValue())
                .like(BaseSysLogEnt::getResponseValue, input.getResponseValue())
                .eq(BaseSysLogEnt::getUserId, input.getUserId())
                .eq(BaseSysLogEnt::getExecutionTime, input.getExecutionTime())
                .like(BaseSysLogEnt::getExceptionInfo, input.getExceptionInfo())
                .eq(BaseSysLogEnt::getIsException, input.getIsException());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseSysLogOutput.class);
    }

    @XApiGet
    public XPageResult<QueryBaseSysLogOutput> dynamicQueryBaseSysLog(XContext context, CustomQueryInput input) {
        BaseSysLogMapper mapper = context.getBean(BaseSysLogMapper.class);
        return XMapperHelper.query(mapper, input, BaseSysLogEnt.class, QueryBaseSysLogOutput.class);
    }

    @XApiGet
    public XSingleResult<GetBaseSysLogViewOutput> getBaseSysLogView(XContext context, GetBaseSysLogViewInput input) {
        BaseSysLogViewMapper mapper = context.getBean(BaseSysLogViewMapper.class);
        BaseSysLogView record = new BaseSysLogView();
        XCopyUtils.copyObject(input, record);
        BaseSysLogView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseSysLogViewOutput output = new GetBaseSysLogViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiGet
    public XPageResult<QueryBaseSysLogViewOutput> queryBaseSysLogView(XContext context, QueryBaseSysLogViewInput input) {
        BaseSysLogViewMapper mapper = context.getBean(BaseSysLogViewMapper.class);
        BaseSysLogView record = new BaseSysLogView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BaseSysLogView> list = mapper.selectList(record);
        PageInfo<BaseSysLogView> pageInfo = new PageInfo<>(list);
        List<QueryBaseSysLogViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseSysLogViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }
}
