package pps.core.common.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.ZipUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.io.Files;
import org.apache.commons.lang3.StringUtils;
import pps.core.common.config.ObsConfig;
import pps.core.common.entity.SysFileAttachmentEnt;
import pps.core.common.enums.FileTypeEnum;
import pps.core.common.mapper.SysFileAttachmentMapper;
import pps.core.common.service.data.file.*;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.HttpTookit;
import pps.core.common.utils.UUIDHelper;
import sun.misc.BASE64Decoder;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.XStartup;
import xstartup.base.pattern.XProviderManager;
import xstartup.base.tool.XStorageTool;
import xstartup.base.util.XFileUtils;
import xstartup.base.util.XUuidUtils;
import xstartup.config.XConfProperties;
import xstartup.config.XConfPropertiesHelper;
import xstartup.core.toolkit.provider.data.GetObjectUrlParam;
import xstartup.core.toolkit.provider.data.GetObjectUrlResult;
import xstartup.core.toolkit.provider.data.SaveObjectParam;
import xstartup.core.toolkit.provider.data.SaveObjectResult;
import xstartup.core.toolkit.provider.spec.XObjectStorageProvider;
import xstartup.data.XFileResult;
import xstartup.data.XFileType;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiUpload;
import xstartup.toolkit.office.helper.XExcelHelper;

import javax.activation.MimetypesFileTypeMap;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import java.io.*;
import java.net.URL;
import java.net.URLEncoder;
import java.util.*;

@XService
public class FileService {

    /**
     * url网络传输
     *
     * @param context
     * @param fileKey
     * @return
     */
    public static XSingleResult getDownloadUrlByfileKey(XContext context, String fileKey) {
        XConfProperties xConfProperties = XConfPropertiesHelper.find(context,
                "x.storage",
                "type",
                new String[]{"endpoint", "access-key-id", "secret-access-key", "bucket-name", "region"},
                XStartup.getCurrent().getName(),
                true);

        context.getLogger().info("配置2信息xConfProperties：" + xConfProperties);

        if (xConfProperties == null) {
            return XSingleResult.error(XError.NotFound.getCode(), "配置信息没有找到");
        }

        XObjectStorageProvider provider = XProviderManager.findImplement(context,
                XObjectStorageProvider.class,
                xConfProperties.getProperty("type"));

        if (provider == null) {
            return XSingleResult.error(XError.NotFound.getCode(), "存储实现没有找到");
        }

        //String fileKey = input.getFileKey();
        GetObjectUrlParam getFileUrlParam = new GetObjectUrlParam();
        getFileUrlParam.setEndpoint(xConfProperties.getProperty("endpoint", null));
        getFileUrlParam.setRegion(xConfProperties.getProperty("region", null));
        getFileUrlParam.setExpires(ObsConfig.urlExpires.getValue(context));
        getFileUrlParam.setBucketName(xConfProperties.getProperty("bucket-name"));
        getFileUrlParam.setFileKey(fileKey);
        getFileUrlParam.setAccessKeyId(xConfProperties.getProperty("access-key-id"));
        getFileUrlParam.setSecretAccessKey(xConfProperties.getProperty("secret-access-key"));
        GetObjectUrlResult getFileUrlServiceResult = provider.getObjectUrl(context, getFileUrlParam).orElseThrow();
        return XSingleResult.success(getFileUrlServiceResult.getUrl());
    }

    /**
     * 下载解决中文文件名乱码问题
     *
     * @param filename 文件名
     * @return 编码后的文件名
     * @throws UnsupportedEncodingException 编码错误
     */
    private static String encode(String filename) throws UnsupportedEncodingException {
        if (filename == null) {
            return "null";
        }
        return URLEncoder.encode(Files.getNameWithoutExtension(filename), "UTF-8") + "." + Files.getFileExtension(filename);
    }

    @XText("上传文件")
    @XApiUpload
    public XSingleResult<FileUploadOutput> uploadFile(XContext context, FileUploadInput input) throws IOException {
        XConfProperties xConfProperties = XConfPropertiesHelper.find(context,
                "x.storage",
                "type",
                new String[]{"endpoint", "access-key-id", "secret-access-key", "bucket-name", "region"},
                XStartup.getCurrent().getName(),
                true);
        if (xConfProperties == null) {
            return XSingleResult.error(XError.NotFound.getCode(), "配置信息没有找到");
        }

        XObjectStorageProvider provider = XProviderManager.findImplement(context,
                XObjectStorageProvider.class,
                xConfProperties.getProperty("type"));

        if (provider == null) {
            return XSingleResult.error(XError.NotFound.getCode(), "存储实现没有找到");
        }
        String contentType = new MimetypesFileTypeMap().getContentType(input.getFile().getFileName());

        SaveObjectParam saveFileParam = new SaveObjectParam();
        saveFileParam.setBucketName(xConfProperties.getProperty("bucket-name"));
        saveFileParam.setAccessKeyId(xConfProperties.getProperty("access-key-id"));
        saveFileParam.setSecretAccessKey(xConfProperties.getProperty("secret-access-key"));
        saveFileParam.setEndpoint(xConfProperties.getProperty("endpoint", null));
        saveFileParam.setRegion(xConfProperties.getProperty("region", null));
        saveFileParam.setInputStream(input.getFile().getInputStream());
        saveFileParam.setContentType(contentType);
        saveFileParam.setFileName(input.getFile().getFileName());
        saveFileParam.setFileKey(UUIDHelper.newUUID());
        SaveObjectResult saveObjectResult = provider.saveObject(context, saveFileParam).orElseThrow();

        SysFileAttachmentMapper mapper = context.getBean(SysFileAttachmentMapper.class);
        SysFileAttachmentEnt entity = new SysFileAttachmentEnt();
        PpsUserSession session = context.getSession(PpsUserSession.class);
        entity.setBsnsId(input.getBsnsId());
        entity.setFolderName(input.getFolderName());
        entity.setFileKey(saveObjectResult.getFileKey());
        entity.setAttachName(input.getFile().getFileName());
        entity.setAttachSize(0);
        entity.setAttachType(input.getFile().getExtension());
        entity.setCreateById(session.getId());
        entity.setCreateByName(session.getUserName());
        entity.setCreateTime(new Date());
        mapper.insert(entity);

        FileUploadOutput uploadFileOutput = new FileUploadOutput();
        uploadFileOutput.setFileKey(saveObjectResult.getFileKey());
        return XSingleResult.success(uploadFileOutput);
    }

    @XText("上传base64")
    @XApiPost
    public XSingleResult<FileUploadOutput> uploadBase64(XContext context, Base64UploadInput input) throws IOException {
        XConfProperties xConfProperties = XConfPropertiesHelper.find(context,
                "x.storage",
                "type",
                new String[]{"endpoint", "access-key-id", "secret-access-key", "bucket-name", "region"},
                XStartup.getCurrent().getName(),
                true);
        if (xConfProperties == null) {
            return XSingleResult.error(XError.NotFound.getCode(), "配置信息没有找到");
        }

        XObjectStorageProvider provider = XProviderManager.findImplement(context,
                XObjectStorageProvider.class,
                xConfProperties.getProperty("type"));

        if (provider == null) {
            return XSingleResult.error(XError.NotFound.getCode(), "存储实现没有找到");
        }

        BASE64Decoder decoder = new BASE64Decoder();
        byte[] bytes = decoder.decodeBuffer(input.getBase64Str());
        InputStream inputStream = new ByteArrayInputStream(bytes);

        String contentType = input.getSuffix();
        if (!contentType.startsWith(".")) {
            contentType = "." + contentType;
        }

        FileTypeEnum byType = FileTypeEnum.findByType(contentType);

        SaveObjectParam saveFileParam = new SaveObjectParam();
        saveFileParam.setBucketName(xConfProperties.getProperty("bucket-name"));
        saveFileParam.setAccessKeyId(xConfProperties.getProperty("access-key-id"));
        saveFileParam.setSecretAccessKey(xConfProperties.getProperty("secret-access-key"));
        saveFileParam.setEndpoint(xConfProperties.getProperty("endpoint", null));
        saveFileParam.setRegion(xConfProperties.getProperty("region", null));
        saveFileParam.setInputStream(inputStream);
        saveFileParam.setContentType(byType.getFileType().getContentType());
        saveFileParam.setFileName(input.getFileName());
        saveFileParam.setFileKey(UUIDHelper.newUUID());
        SaveObjectResult saveObjectResult = provider.saveObject(context, saveFileParam).orElseThrow();

        SysFileAttachmentMapper mapper = context.getBean(SysFileAttachmentMapper.class);
        SysFileAttachmentEnt entity = new SysFileAttachmentEnt();
        PpsUserSession session = context.getSession(PpsUserSession.class);
        entity.setBsnsId(input.getBsnsId());
        entity.setFolderName(input.getFolderName());
        entity.setFileKey(saveObjectResult.getFileKey());
        entity.setAttachName(input.getFileName());
        entity.setAttachSize(0);
        entity.setAttachType(input.getSuffix());
        entity.setCreateById(session.getId());
        entity.setCreateByName(session.getUserName());
        entity.setCreateTime(new Date());
        mapper.insert(entity);

        FileUploadOutput uploadFileOutput = new FileUploadOutput();
        uploadFileOutput.setFileKey(saveObjectResult.getFileKey());
        return XSingleResult.success(uploadFileOutput);
    }

    @XApiGet
    @XText("获取文件地址")
    public XSingleResult getDownloadUrl(XContext context, FileDownloadInput input) {
        XConfProperties xConfProperties = XConfPropertiesHelper.find(context,
                "x.storage",
                "type",
                new String[]{"endpoint", "access-key-id", "secret-access-key", "bucket-name", "region"},
                XStartup.getCurrent().getName(),
                true);

        context.getLogger().info("配置1信息xConfProperties：" + xConfProperties);

        if (xConfProperties == null) {
            return XSingleResult.error(XError.NotFound.getCode(), "配置信息没有找到");
        }

        XObjectStorageProvider provider = XProviderManager.findImplement(context,
                XObjectStorageProvider.class,
                xConfProperties.getProperty("type"));

        if (provider == null) {
            return XSingleResult.error(XError.NotFound.getCode(), "存储实现没有找到");
        }

        String fileKey = input.getFileKey();
        GetObjectUrlParam getFileUrlParam = new GetObjectUrlParam();
        getFileUrlParam.setEndpoint(xConfProperties.getProperty("endpoint", null));
        getFileUrlParam.setRegion(xConfProperties.getProperty("region", null));
        getFileUrlParam.setExpires(ObsConfig.urlExpires.getValue(context));
        getFileUrlParam.setBucketName(xConfProperties.getProperty("bucket-name"));
        getFileUrlParam.setFileKey(fileKey);
        getFileUrlParam.setAccessKeyId(xConfProperties.getProperty("access-key-id"));
        getFileUrlParam.setSecretAccessKey(xConfProperties.getProperty("secret-access-key"));
        GetObjectUrlResult getFileUrlServiceResult = provider.getObjectUrl(context, getFileUrlParam).orElseThrow();
        return XSingleResult.success(getFileUrlServiceResult.getUrl());
    }

    @XApiGet
    @XText("下载文件")
    public XFileResult downloadFile(XContext context, FileDownloadInput input) {
        //获取文件信息
        SysFileAttachmentMapper mapper = context.getBean(SysFileAttachmentMapper.class);
        QueryWrapper<SysFileAttachmentEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysFileAttachmentEnt::getFileKey, input.getFileKey());
        SysFileAttachmentEnt sysFileAttachmentEnt = mapper.selectOne(queryWrapper);
        if (sysFileAttachmentEnt == null) {
            return XFileResult.error(context, XError.NotFound);
        }

        String attachType = sysFileAttachmentEnt.getAttachType();
        if (StringUtils.isBlank(attachType)) {
            return XFileResult.error(1000, "文件格式有误，请重新上传");
        }

        if (!attachType.startsWith(".")) {
            attachType = "." + attachType;
        }
        if (sysFileAttachmentEnt.getAttachName().endsWith(attachType)) {
            attachType = "";
        }

        String tempFile = XStorageTool.getAbsolutePath("/temp/file/" + sysFileAttachmentEnt.getAttachName() + attachType);
        XSingleResult result = getDownloadUrl(context, input);
        result.throwIfFail();
        String downloadUrl = (String) result.getResult();

        HttpsURLConnection connection = null;
        InputStream inputStream = null;
        FileOutputStream outputStream = null;
        try {
            URL url = new URL(downloadUrl);
            HttpsURLConnection.setDefaultHostnameVerifier(HttpTookit.ignoreHostnameVerifier);
            connection = (HttpsURLConnection) url.openConnection();
            // Prepare SSL Context
            TrustManager[] tm = {HttpTookit.ignoreCertificationTrustManger};
            SSLContext sslContext = SSLContext.getInstance("SSL", "SunJSSE");
            sslContext.init(null, tm, new java.security.SecureRandom());

            // 从上述SSLContext对象中得到SSLSocketFactory对象
            SSLSocketFactory ssf = sslContext.getSocketFactory();
            connection.setSSLSocketFactory(ssf);

            inputStream = connection.getInputStream();

            byte[] buff = new byte[1024];
            File file = new File(tempFile);
            XFileUtils.deleteFile(file);
            if (!file.getParentFile().exists()) {
                file.getParentFile().mkdirs();
            }
            outputStream = new FileOutputStream(file);
            while (inputStream.read(buff) != -1) {
                outputStream.write(buff, 0, buff.length);
                outputStream.flush();
            }
        } catch (Exception e) {
            context.getLogger().error("输出文件流失败：" + e.getMessage());
            e.printStackTrace();
        } finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (outputStream != null) {
                try {
                    outputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (null != connection) {
                connection.disconnect();
            }
            context.getLogger().info("数据流处理完毕");
        }

        return XFileResult.success(FileTypeEnum.findByType(attachType).getFileType(), tempFile);
    }

    @XText("导入demo")
    @XApiUpload
    public XServiceResult importFile(XContext context, FileUploadInput input) {
        InputStream inputStream = input.getFile().getInputStream();
        //todo 解析文件，处理业务
        return XServiceResult.OK;
    }

    @XText("导出demo")
    @XApiGet(anonymous = true)
    public XFileResult exportExcel(XContext context, ExportExcelInput input) {
        String tempFile = XStorageTool.getAbsolutePath("/temp/excel/" + XUuidUtils.randomUUID() + ".xlsx");
        List<ExcelModel> list = new ArrayList<>();

        for (int index = 0; index < 10; index++) {
            ExcelModel excelModel = new ExcelModel();
            excelModel.setText("测试" + index);
            list.add(excelModel);
        }
        XExcelHelper.write(context, tempFile, "excel模版", list, ExcelModel.class);
        return XFileResult.success(XFileType.APPLICATION_XLSX, tempFile);
    }

    @XText("批量下载")
    @XApiGet
    public XFileResult downloadFileZip(XContext context, FileBatchDownloadInput input) throws IOException {

        if (CollectionUtil.isEmpty(input.getFileKeyList())) {
            return XFileResult.error(1000, "需要压缩的附件fileKeyList不能为空");
        }
        SysFileAttachmentMapper mapper = context.getBean(SysFileAttachmentMapper.class);
        QueryWrapper<SysFileAttachmentEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().in(SysFileAttachmentEnt::getFileKey, input.getFileKeyList());
        List<SysFileAttachmentEnt> ent_list = mapper.selectList(queryWrapper);
        if (CollectionUtil.isEmpty(ent_list)) {
            return XFileResult.error(context, XError.NotFound);
        }
        Set<String> set = new HashSet();
        for (SysFileAttachmentEnt sysFileAttachmentEnt : ent_list) {
            String attachType = sysFileAttachmentEnt.getAttachType();
            if (StringUtils.isBlank(attachType)) {
                context.getLogger().error("该文件格式有误：" + sysFileAttachmentEnt.getAttachName());
            }
            if (!attachType.startsWith(".")) {
                attachType = "." + attachType;
            }
            if (sysFileAttachmentEnt.getAttachName().endsWith(attachType)) {
                attachType = "";
            }
            //接收 obs取出附件后的 File路径
            String tempFile = XStorageTool.getAbsolutePath("/temp/file/" + sysFileAttachmentEnt.getAttachName() + attachType);
            context.getLogger().info("附件sysFileAttachmentEnt：" + sysFileAttachmentEnt.getFileKey());
            FileDownloadInput fileDownloadInput = new FileDownloadInput();
            fileDownloadInput.setFileKey(sysFileAttachmentEnt.getFileKey());
            XSingleResult result = getDownloadUrl(context, fileDownloadInput);
            result.throwIfFail();
            String downloadUrl = (String) result.getResult();
            HttpsURLConnection connection = null;
            InputStream inputStream = null;
            FileOutputStream outputStream = null;
            try {
                URL url = new URL(downloadUrl);
                HttpsURLConnection.setDefaultHostnameVerifier(HttpTookit.ignoreHostnameVerifier);
                connection = (HttpsURLConnection) url.openConnection();
                TrustManager[] tm = {HttpTookit.ignoreCertificationTrustManger};
                SSLContext sslContext = SSLContext.getInstance("SSL", "SunJSSE");
                sslContext.init(null, tm, new java.security.SecureRandom());
                SSLSocketFactory ssf = sslContext.getSocketFactory();
                connection.setSSLSocketFactory(ssf);
                //通过obs的url网络传输 得到附件输入流
                inputStream = connection.getInputStream();

                byte[] buff = new byte[1024];
                File file = new File(tempFile);
                XFileUtils.deleteFile(file);
                if (!file.getParentFile().exists()) {
                    file.getParentFile().mkdirs();
                }
                outputStream = new FileOutputStream(file);
                while (inputStream.read(buff) != -1) {
                    outputStream.write(buff, 0, buff.length);
                    outputStream.flush();
                }
            } catch (Exception e) {
                context.getLogger().error("输出文件流失败：" + e.getMessage());
                e.printStackTrace();
            } finally {
                if (inputStream != null) {
                    try {
                        inputStream.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
                if (outputStream != null) {
                    try {
                        outputStream.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
                if (null != connection) {
                    connection.disconnect();
                }
                context.getLogger().info("数据流处理完毕");
            }

            set.add(tempFile);
        }
        context.getLogger().info("附件set集合：" + set);

        String tempUrl = System.getProperty("java.io.tmpdir") + "/temp/file/zip/";
        String tempFilePath = tempUrl + encode(input.getFolderName()) + ".zip";
        try {
            int index = 0;
            File[] srcFiles = new File[set.size()];
            for (String path : set) {
                srcFiles[index++] = new File(path);
            }
            context.getLogger().info("File数组：" + srcFiles);
            //zip压缩
            ZipUtil.zip(FileUtil.file(tempFilePath), true, srcFiles);
            context.getLogger().info("zip压缩包的路径：" + tempFilePath);
        } catch (Exception e) {
            context.getLogger().error("zip压缩失败：" + e.getMessage());
        }

        return XFileResult.success(FileTypeEnum.findByType(".zip").getFileType(), tempFilePath);
    }
}
