package pps.core.common.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.util.CollectionUtils;
import pps.core.common.entity.SysFileAttachmentEnt;
import pps.core.common.entity.SysFileMiddleAttachmentEnt;
import pps.core.common.mapper.SysFileAttachmentMapper;
import pps.core.common.mapper.SysFileMiddleAttachmentMapper;
import pps.core.common.service.data.sys_file_middle_attachment.CreateSysFileMiddleAttachmentInput;
import pps.core.common.service.data.sys_file_middle_attachment.DeleteSysFileMiddleAttachmentInput;
import pps.core.common.service.data.sys_file_middle_attachment.GetSysFileMiddleAttachmentInput;
import pps.core.common.service.data.sys_file_middle_attachment.GetSysFileMiddleAttachmentOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.feature.api.annotation.XApiPost;

import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@XText("附件中间表接口")
@XService
public class SysFileMiddleAttachmentService {

    @XText("删除中间附件")
    @XApiPost
    public XServiceResult deleteSysFileMiddleAttachmentByBsnsId(XContext context, DeleteSysFileMiddleAttachmentInput input) {
        SysFileMiddleAttachmentMapper mapper = context.getBean(SysFileMiddleAttachmentMapper.class);
        QueryWrapper<SysFileMiddleAttachmentEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysFileMiddleAttachmentEnt::getBsnsId, input.getBsnsId());
        List<SysFileMiddleAttachmentEnt> list = mapper.selectList(queryWrapper);
        if (!CollectionUtils.isEmpty(list)) {
            List<String> ids = list.stream().map(SysFileMiddleAttachmentEnt::getId).collect(Collectors.toList());
            mapper.deleteBatchIds(ids);
        }
        return XServiceResult.OK;
    }

    @XText("添加中间附件")
    @XApiPost
    public XServiceResult createSysFileMiddleAttachment(XContext context, CreateSysFileMiddleAttachmentInput input) {
        SysFileMiddleAttachmentMapper mapper = context.getBean(SysFileMiddleAttachmentMapper.class);
        List<String> fileKeyList = input.getFileKeyList();
        SysFileAttachmentMapper attachmentMapper = context.getBean(SysFileAttachmentMapper.class);
        for (String fileKey : fileKeyList) {
            QueryWrapper<SysFileAttachmentEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysFileAttachmentEnt::getFileKey, fileKey);
            SysFileAttachmentEnt attachmentEnt = attachmentMapper.selectOne(queryWrapper);
            if (attachmentEnt == null) {
                continue;
            }
            SysFileMiddleAttachmentEnt entity = XCopyUtils.copyNewObject(attachmentEnt, SysFileMiddleAttachmentEnt.class);
            entity.setId(UUID.randomUUID().toString());
            entity.setBsnsId(input.getBsnsId());
            entity.setBsnsType(input.getBsnsType());
            entity.setFileKey(fileKey);
            mapper.insert(entity);
        }
        return XServiceResult.OK;
    }

    @XText("获取业务附件")
    @XApiPost
    public XListResult<GetSysFileMiddleAttachmentOutput> getSysFileAttachmentList(XContext context, GetSysFileMiddleAttachmentInput input) {
        SysFileMiddleAttachmentMapper mapper = context.getBean(SysFileMiddleAttachmentMapper.class);
        QueryWrapper<SysFileMiddleAttachmentEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysFileMiddleAttachmentEnt::getBsnsId, input.getBsnsId());
        List<SysFileMiddleAttachmentEnt> list = mapper.selectList(queryWrapper);
        List<GetSysFileMiddleAttachmentOutput> outputs = XCopyUtils.copyNewList(list, GetSysFileMiddleAttachmentOutput.class);
        return XListResult.success(outputs);
    }


}
