package pps.core.common.utils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.regex.Pattern;

/**
 * Description:
 * date: 2020/11/18 16:47
 *
 * @author zhujiangwei
 */
public class BigDecimalUtil {

    /**
     * 小数格式化
     *
     * @param obj 原数据
     * @return
     */
    public static String parseBigDecimal2Str(Object obj) {
        return removeLastZero(String.valueOf(parseBigDecimal(obj, 10)));
    }

    /**
     * 小数格式化
     *
     * @param obj 原数据
     * @return
     */
    public static double parseBigDecimal2Double(Object obj) {
        return Double.parseDouble(removeLastZero(String.valueOf(parseBigDecimal(obj, 10))));
    }


    /**
     * 小数格式化(保留固定的小数位数)
     *
     * @param obj 原数据
     * @param num 保留小数的位数
     * @return
     */
    public static String parseBigDecimal2Str(Object obj, int num) {
        return String.valueOf(parseBigDecimal(obj, num));
    }

    public static BigDecimal parseBigDecimal(Object obj, int num) {
        if (obj == null) return BigDecimal.ZERO.setScale(num, BigDecimal.ROUND_HALF_UP);
        BigDecimal org = null;
        if (obj instanceof BigDecimal) {
            org = (BigDecimal) obj;
        } else {
            String pattern = "([1-9]\\d*\\.?\\d*)|(0\\.\\d*[1-9])";
            if (!Pattern.matches(pattern, String.valueOf(obj))) { //非数字字符串
                org = BigDecimal.ZERO;
            } else {
                org = parseObj2BigDecimal(obj);
            }
        }
        return org.setScale(num, BigDecimal.ROUND_HALF_UP);
    }

    /**
     * 获取百分比字符串
     *
     * @param total
     * @param part
     * @return
     */
    public static String getPercentStr(int total, int part) {
        if (total == 0) return "100%";
        if (part == 0) return "0%";
        return new BigDecimal(part).divide(new BigDecimal(total), 2, RoundingMode.HALF_UP).multiply(new BigDecimal(100)) + "%";
    }

    /**
     * 空值转换
     */
    public static BigDecimal parseNull2Zero(BigDecimal org) {
        return org == null ? BigDecimal.ZERO : org;
    }

    /**
     * 去掉数字字符串末尾的0
     */
    public static String removeLastZero(String org) {
        if (org == null || "".equals(org.trim())) return "0";
        //String pattern = "(-?([1-9]\\d*\\.?\\d*)|(0\\.\\d*[1-9]))";
        //if(!Pattern.matches(pattern,org)) return "0"; //非数字字符串
        DecimalFormat format = new DecimalFormat("###############.##########");
        return format.format(new Double(org));
    }

    public static BigDecimal parseObj2BigDecimal(Object obj) {
        if (obj == null) return BigDecimal.ZERO;
        String str = String.valueOf(obj).trim();
        if (str.length() == 0) return BigDecimal.ZERO;
        BigDecimal result = null;
        try {
            result = new BigDecimal(str);
        } catch (Exception e) {
            e.printStackTrace();
            result = BigDecimal.ZERO;
        }
        return result;
    }
}
