package pps.core.common.utils;

import com.google.common.base.Strings;
import com.google.common.collect.ImmutableList;

import java.util.ArrayList;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicLong;

/*
 * 计数构建器
 * */
public class CounterBuilder {
    public static final CounterBuilder globalCounterBuilder = new CounterBuilder();
    public static final String DEPLOY_KEY = "__DEPLOY__";
    public static final String COUNTER_NAME_INSTANCE = "__INSTANCE__";
    public static final String COUNTER_NAME_OWNER = "__OWNER__";
    public static final String COUNTER_NAME_ENV = "__ENV__";
    public static final String COUNTER_NAME_BUILDTIME = "__BUILDTIME__";
    private final static ConcurrentHashMap<String, CounterBuilder> counterBuilders = new ConcurrentHashMap<>();
    public final String counterId = UUID.randomUUID().toString();
    //    private final ReentrantReadWriteLock  lock = new ReentrantReadWriteLock();
    private final ConcurrentHashMap<String, String> fieldValueMap = new ConcurrentHashMap<String, String>();
    private final ConcurrentHashMap<String, AtomicLong> counterMap = new ConcurrentHashMap<>();
    private final ConcurrentHashMap<String, CounterBuildItemInfoType> fileInfoTypeMap = new ConcurrentHashMap<>();
    private String groupName;
    public CounterBuilder() {
        this.setFieldValue("__COUNTERID__", counterId);
        this.setFieldValue("__CREATETIME__", DateUtil.getCurrentDateTime());
    }

    public CounterBuilder(String groupName) {
        this.groupName = groupName;
    }

    public static String getInstanceId() {
        if (globalCounterBuilder.fieldValueMap.containsKey(COUNTER_NAME_INSTANCE)) {
            return globalCounterBuilder.fieldValueMap.get(COUNTER_NAME_INSTANCE);
        } else {
            return "";
        }
    }

    public static CounterBuilder getCounterBuilder(String counterName) {
        if (counterBuilders.containsKey(counterName)) {
            return counterBuilders.get(counterName);
        } else {
            CounterBuilder counterBuilder = new CounterBuilder();
            counterBuilders.put(counterName, counterBuilder);
            return counterBuilder;
        }
    }

    public ConcurrentHashMap<String, String> getFieldValueMap() {
        return fieldValueMap;
    }

    public ConcurrentHashMap<String, AtomicLong> getCounterMap() {
        return counterMap;
    }

    public String getGroupName() {
        return groupName;
    }

    public void setGroupName(String groupName) {
        this.groupName = groupName;
    }

    public CounterBuilder addCounter(Throwable throwable) {
        return this.addCounter("Exception-Total")
                .addCounter("Exception-" + throwable.getClass().getSimpleName(), 1, CounterBuildItemInfoType.ERROR);
    }

    public CounterBuilder addCounter(Throwable throwable, Class<?> reporter) {
        return this.addCounter("Exception-Total")
                .addCounter("Exception-" + throwable.getClass().getSimpleName())
                .addCounter(reporter.getSimpleName() + "-" + throwable.getClass().getSimpleName(), 1, CounterBuildItemInfoType.ERROR);
    }

    public CounterBuilder addCounter(String counterName) {
        return this.addCounter(counterName, 1, detectInfoTypeByCounterName(counterName));
    }

    public CounterBuilder addCounter(String counterName, Class<?> reporter) {
        return this.addCounter(reporter.getSimpleName() + "-" + counterName, 1, detectInfoTypeByCounterName(counterName));
    }

    public CounterBuilder addCounter(String counterName, Class<?> reporter, CounterBuildItemInfoType infoType) {
        return addCounter(reporter.getSimpleName() + "-" + counterName, 1L, infoType);
    }

    public CounterBuilder addCounter(String counterName, CounterBuildItemInfoType infoType) {
        return addCounter(counterName, 1L, infoType);
    }

    public CounterBuilder addCounter(String counterName, long v) {
        return this.addCounter(counterName, v, detectInfoTypeByCounterName(counterName));
    }

    public CounterBuilder addCounter(String counterName, long v, CounterBuildItemInfoType infoType) {
        if (counterName == null || counterName.isEmpty()) {
            return this;
        }

        if (!Strings.isNullOrEmpty(this.groupName)) {
            counterName = String.format("%s-%s", this.groupName, counterName);
        }

        if (this.counterMap.containsKey(counterName)) {
            this.counterMap.get(counterName).getAndAdd(v);
            return this;
        }

        if (this.counterMap.containsKey(counterName)) {
            this.counterMap.get(counterName).getAndAdd(v);
            return this;
        }

        this.counterMap.put(counterName, new AtomicLong(v));
        this.fileInfoTypeMap.put(counterName, infoType);
        return this;
    }

    public CounterBuilder setCounter(String counterName, long v) {
        return this.setCounter(counterName, v, detectInfoTypeByCounterName(counterName));
    }

    public CounterBuilder setCounter(String counterName, long newCounterValue, CounterBuildItemInfoType infoType) {
        if (counterName == null || counterName.isEmpty()) {
            return this;
        }

        if (!Strings.isNullOrEmpty(this.groupName)) {
            counterName = String.format("%s-%s", this.groupName, counterName);
        }

        if (this.fieldValueMap.containsKey(counterName)) {
            this.counterMap.get(counterName).set(newCounterValue);
            return this;
        }
        if (this.fieldValueMap.containsKey(counterName)) {
            this.counterMap.get(counterName).set(newCounterValue);
            return this;
        }

        this.counterMap.put(counterName, new AtomicLong(newCounterValue));
        this.fileInfoTypeMap.put(counterName, infoType);

        return this;
    }

    public CounterBuilder setFieldValue(String fieldName, String fieldValue) {
        return setFieldValue(fieldName, fieldValue, detectInfoTypeByCounterName(fieldName));
    }

    public CounterBuilder setFieldValue(String fieldName, String fieldValue, CounterBuildItemInfoType infoType) {
        if (fieldName == null || fieldName.isEmpty()) {
            return this;
        }

        if (!Strings.isNullOrEmpty(this.groupName)) {
            fieldName = String.format("%s-%s", this.groupName, fieldName);
        }

        this.fieldValueMap.put(fieldName, fieldValue);
        this.fileInfoTypeMap.put(fieldName, infoType);
        return this;
    }

    private CounterBuildItemInfoType detectInfoTypeByCounterName(String counterName) {
        if (counterName.contains("Error")
                || counterName.contains("error")
                || counterName.contains("ERROR")
                || counterName.contains("Exception")) {
            return CounterBuildItemInfoType.ERROR;
        }
        return CounterBuildItemInfoType.NORNAL;
    }

    public long getCounter(String counterName) {
        if (counterName == null || counterName.isEmpty()) {
            return 0;
        }

        if (!Strings.isNullOrEmpty(this.groupName)) {
            counterName = String.format("%s-%s", this.groupName, counterName);
        }

        if (!this.counterMap.containsKey(counterName)) {
            return 0;
        } else {
            return this.counterMap.get(counterName).get();
        }
    }

    public String getFieldValue(String fieldName) {
        if (fieldName == null || fieldName.isEmpty()) {
            return "";
        }

        if (!Strings.isNullOrEmpty(this.groupName)) {
            fieldName = String.format("%s-%s", this.groupName, fieldName);
        }

        return this.fieldValueMap.getOrDefault(fieldName, "");//[fieldName];
    }

    @Override
    public String toString() {
        ArrayList<String> lines = new ArrayList<String>(this.fieldValueMap.size() + this.counterMap.size());

        StringBuilder stringBuilder = new StringBuilder();

        ImmutableList<Map.Entry<String, String>> fieldsList = ImmutableList.copyOf(this.fieldValueMap.entrySet());
        fieldsList.forEach(item -> {
            lines.add(String.format("%s : %s", item.getKey(), item.getValue()));
        });
        ImmutableList<Map.Entry<String, AtomicLong>> fieldsList2 = ImmutableList.copyOf(this.counterMap.entrySet());
        fieldsList2.forEach(item -> {
            lines.add(String.format("%s : %d", item.getKey(), item.getValue().get()));
        });
        for (String line : lines) {
            stringBuilder
                    .append(line)
                    .append(System.getProperty("line.separator"))
            ;
        }

        return stringBuilder.toString();
    }

    public DynObj toDynObj() {
        DynObj obj = new DynObj();
        ImmutableList<Map.Entry<String, String>> fieldsList = ImmutableList.copyOf(this.fieldValueMap.entrySet());
        fieldsList.forEach(item -> {
            obj.put(item.getKey(), item.getValue());
        });
        ImmutableList<Map.Entry<String, AtomicLong>> fieldsList2 = ImmutableList.copyOf(this.counterMap.entrySet());
        fieldsList2.forEach(item -> {
            obj.put(item.getKey(), item.getValue().get());
        });
        return obj;
    }

    public enum CounterBuildItemInfoType {
        NORNAL,
        ERROR;
    }

}
