package pps.core.common.utils;

import com.google.common.base.Strings;

import java.util.concurrent.ConcurrentHashMap;

public class TimeSpanChecker {
    public final static int TIMESPAN_MS_5s = 5 * 1000;
    public final static int TIMESPAN_MS_10s = 10 * 1000;
    public final static int TIMESPAN_MS_15s = 15 * 1000;
    public final static int TIMESPAN_MS_30s = 30 * 1000;
    public final static int TIMESPAN_MS_1m = 60 * 1000;
    public final static int TIMESPAN_MS_5m = 5 * 60 * 1000;
    public final static int TIMESPAN_MS_10m = 10 * 60 * 1000;
    public final static int TIMESPAN_MS_30m = 30 * 60 * 1000;
    public final static int TIMESPAN_MS_1h = 60 * 60 * 1000;
    public final static int TIMESPAN_MS_1d = 24 * 60 * 60 * 1000;
    private static final ConcurrentHashMap<String, TimeSpanChecker> checkers = new ConcurrentHashMap<>();
    private final boolean skipFirstCheck;
    private long preCheckTimeMs = 0;
    private int seconds = 5;

    public TimeSpanChecker(int _seconds) {
        this.seconds = _seconds;
        this.skipFirstCheck = true;
    }

//    public Date getPreCheckTimeMs() {
//        return preCheckTimeMs;
//    }

    public TimeSpanChecker(int _seconds, boolean _skipFirstCheck) {
        this.seconds = _seconds;
        this.skipFirstCheck = _skipFirstCheck;
    }

    public static int getChecksSize() {
        return checkers.size();
    }

    public static boolean checkNow(String checkerName, int seconds) {
        return checkNow(checkerName, seconds, true);
    }

    public static boolean checkNow(String checkerName, int seconds, boolean skipFirstCheck) {
        if (Strings.isNullOrEmpty(checkerName) || seconds <= 0) {
            return false;
        }

        if (checkers.containsKey(checkerName)) {
            checkers.put(checkerName, new TimeSpanChecker(seconds, skipFirstCheck));
        }

        return checkers.get(checkerName).checkNow();
    }

    public static double timeUsed(Runnable runnable) {
        long start = System.currentTimeMillis();
        runnable.run();
        long end = System.currentTimeMillis();
        return (end - start) / 1000.0;
    }

    public boolean isSkipFirstCheck() {
        return skipFirstCheck;
    }

    public int getSeconds() {
        return seconds;
    }

    public void setSeconds(int seconds) {
        this.seconds = seconds;
    }

    public boolean checkNow() {
        if (preCheckTimeMs == 0) {
            if (skipFirstCheck) {
                preCheckTimeMs = System.currentTimeMillis();
                return false;
            } else {
                preCheckTimeMs = System.currentTimeMillis();
                return true;
            }
        }

        double ms = System.currentTimeMillis() - this.preCheckTimeMs;
        if (ms >= this.seconds * 1000) {
            preCheckTimeMs = System.currentTimeMillis();
            return true;
        }

        return false;
    }
}
