package pps.core.system.provider.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.base.Splitter;
import com.google.common.base.Strings;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.SystemUserRoleRelService;
import pps.cloud.system.service.data.*;
import pps.core.common.constant.UserConstant;
import pps.core.common.session.MediaTypes;
import pps.core.common.session.PpsUserSession;
import pps.core.common.session.SysOrganization;
import pps.core.system.config.LoginConfig;
import pps.core.system.constant.OuLevelConstant;
import pps.core.system.entity.SysAuthOuScopeView;
import pps.core.system.entity.SysMenuView;
import pps.core.system.entity.SysOrganizationRelEnt;
import pps.core.system.entity.SysUserEnt;
import pps.core.system.mapper.SysAuthOuScopeViewMapper;
import pps.core.system.mapper.SysMenuViewMapper;
import pps.core.system.mapper.SysOrganizationRelMapper;
import pps.core.system.mapper.SysUserMapper;
import xstartup.annotation.XImplement;
import xstartup.base.XContext;
import xstartup.base.XUser;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.core.base.provider.spec.XSessionInjectProvider;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;

import java.util.*;
import java.util.stream.Collectors;

import static pps.core.common.constant.LoginConstant.LOGIN_FLAG_SIMULATE;

@XImplement
public class PpsUserSessionInjectProviderImpl implements XSessionInjectProvider {
    @Override
    public XSingleResult<Object> injectSession(XContext context, XUser user) {
        //todo:注入session数据
        context.log().info("inject session, userCode:{}", user.getCode());
        SysUserMapper mapper = context.getBean(SysUserMapper.class);
        QueryWrapper<SysUserEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserEnt::getId, user.getCode());
        SysUserEnt sysUserEnt = mapper.selectOne(queryWrapper);
        PpsUserSession session = new PpsUserSession();
        XCopyUtils.copyObject(sysUserEnt, session);

        //获取页面授权信息
        SysAuthOuScopeViewMapper scopeViewMapper = context.getBean(SysAuthOuScopeViewMapper.class);
        SysAuthOuScopeView view = new SysAuthOuScopeView();
        view.setUserId(user.getCode());
        List<SysAuthOuScopeView> sysAuthOuScopeViews = scopeViewMapper.selectList(view);
        Map<String, List<SysAuthOuScopeView>> collect = sysAuthOuScopeViews.stream().collect(Collectors.groupingBy(SysAuthOuScopeView::getMenuId));
        Iterator<Map.Entry<String, List<SysAuthOuScopeView>>> iterator = collect.entrySet().iterator();

        Map<String, List<SysOrganization>> pageOrgMap = new HashMap<>();
        while (iterator.hasNext()) {
            Map.Entry<String, List<SysAuthOuScopeView>> next = iterator.next();
            pageOrgMap.put(next.getKey(), XCopyUtils.copyNewList(next.getValue(), SysOrganization.class));
        }
        session.setPageOrgMap(pageOrgMap);

        GetSysOrganizationViewInput input = new GetSysOrganizationViewInput();
        input.setUserId(session.getId());
        XSingleResult<GetSysOrganizationViewOutput> orgInfo = context.getBean(SysOrganizationCloudService.class).getOuByUserId(context, input);
        session.setOuId(orgInfo.getResult().getOuId());
        session.setOuPath(orgInfo.getResult().getOuPath());
        session.setOuName(orgInfo.getResult().getOuName());
        session.setOuPath(orgInfo.getResult().getOuPath());
        session.setOuLevel(orgInfo.getResult().getOuLevel());
        // 2022-09-29 15:14:47, 查询用户所属的非机关组织机构
        GetSysOrganizationViewOutput ownerOrg = findOwnerOrg(context, orgInfo.getResult());
        if (ownerOrg != null) {
            session.setOwnerOrgId(ownerOrg.getOuId());
            if (Strings.isNullOrEmpty(ownerOrg.getOuId())) {
                session.setOwnerOrgId(ownerOrg.getId());
            }
            session.setOwnerOrgName(ownerOrg.getOuName());
            session.setOwnerOrgPath(ownerOrg.getOuPath());
            session.setOwnerOrgLevel(ownerOrg.getOuLevel());
            if (Strings.isNullOrEmpty(ownerOrg.getOuPath())) {
                //路径为空 查询
                SysOrganizationRelMapper bean = context.getBean(SysOrganizationRelMapper.class);
                QueryWrapper<SysOrganizationRelEnt> qw = new QueryWrapper<SysOrganizationRelEnt>();
                qw.lambda().eq(SysOrganizationRelEnt::getOuId, session.getOwnerOrgId());
                qw.lambda().le(SysOrganizationRelEnt::getEffectTime, new Date());
                qw.lambda().ge(SysOrganizationRelEnt::getEndTime, new Date());
                List<SysOrganizationRelEnt> sysOrganizationRelEnts = bean.selectList(qw);
                if (!sysOrganizationRelEnts.isEmpty()) {
                    session.setOwnerOrgPath(sysOrganizationRelEnts.get(0).getOuIdPath());
                }
            }
        }
        //获取当前用户所属组织机构的介质类型和当前用户授权的组织机构（最终获取组织机构的介质类型）
        if (XStringUtils.isNotEmpty(session.getId())) {
            SysAuthOuScopeView scopeView = new SysAuthOuScopeView();
            scopeView.setUserId(session.getId());
            scopeView.setEndTime(new Date());
            SysAuthOuScopeView mediaTypesView = scopeViewMapper.selectOuMediaTypeByUserId(scopeView);
            Set<String> set = new HashSet<>();
            if (mediaTypesView != null) {
                String mediaType = mediaTypesView.getMediaType();
                if (XStringUtils.isNotEmpty(mediaType)) {
                    String[] split = mediaType.split(",");
                    set.addAll(Arrays.asList(split));
                }
            }
            if (!CollectionUtils.isEmpty(set)) {
                SystemDictionaryService dictService = context.getBean(SystemDictionaryService.class);
                CreateSysDictionaryInput mediaTypesInput = new CreateSysDictionaryInput();
                mediaTypesInput.setAlias("MEDIUM_TYPE");
                XListResult<QuerySysDictionaryViewOutput> mediaTypes = dictService.querySysDictionarysByAlias(context, mediaTypesInput);


                List<String> menuMediaTypes = new ArrayList<>();
                //通过用户角色加载当前用户可操作的介质类型
                //获取当前用户所属角色绑定菜单的介质类型（最终获取菜单的介质类型）
                SysMenuViewMapper menuViewMapper = context.getBean(SysMenuViewMapper.class);
                SysMenuView menuView = new SysMenuView();
                menuView.setUserId(session.getId());
                menuView.setOuId(session.getOuId());
                List<SysMenuView> menuViews = menuViewMapper.selectMediumTypeByUserId(menuView);
                for (SysMenuView sysMenuView : menuViews) {
                    menuMediaTypes.add(sysMenuView.getMediaType());
                }
                //根据菜单的介质类型过滤组织机构的介质类型，最终只显示菜单的介质类型和组织机构的介质类型是一致的。
                //如果菜单中有全介质的话就把组织机构的所有介质类型都展示
                if (!menuMediaTypes.contains("ALL")) {
                    set = set.stream().filter(menuMediaTypes::contains).collect(Collectors.toSet());
                }

                List<MediaTypes> list = new ArrayList<>();
                for (String media : set) {
                    if ("ALL".equals(media)) {
                        list.clear();
                        for (QuerySysDictionaryViewOutput dict : mediaTypes.getResult()) {
                            if (!dict.getDicKey().equals("ALL")) {
                                MediaTypes mediaType = new MediaTypes();
                                mediaType.setMediaTypeCode(dict.getDicKey());
                                mediaType.setMediaTypeName(dict.getDicName());
                                if (dict.getSort() == null) {
                                    mediaType.setSort(999);
                                } else {
                                    mediaType.setSort(dict.getSort());
                                }
                                list.add(mediaType);
                            }
                        }
                        break;
                    } else {
                        for (QuerySysDictionaryViewOutput dict : mediaTypes.getResult()) {
                            if (dict.getDicKey().equals(media)) {
                                MediaTypes mediaType = new MediaTypes();
                                mediaType.setMediaTypeCode(media);
                                mediaType.setMediaTypeName(dict.getDicName());
                                if (dict.getSort() == null) {
                                    mediaType.setSort(999);
                                } else {
                                    mediaType.setSort(dict.getSort());
                                }
                                list.add(mediaType);
                                break;
                            }
                        }
                    }
                }
                list = list.stream().sorted(Comparator.comparing(MediaTypes::getSort)).collect(Collectors.toList());
                list = list.stream().distinct().collect(Collectors.toList());
                session.setMediaTypes(list);
            }


        }

        if (user.getOrgId() == LOGIN_FLAG_SIMULATE) {
            session.setDevops(1);
        } else {
            SystemUserRoleRelService roleRelService = context.getBean(SystemUserRoleRelService.class);
            GetSysUserRoleRelInput roleRelInput = new GetSysUserRoleRelInput();
            roleRelInput.setUserId(session.getId());
            XListResult<GetSysUserRoleRelOutput> roleRelResult = roleRelService.getSysRoleByUserId(context, roleRelInput);
            roleRelResult.throwIfFail();
            List<String> roleList = roleRelResult.getResult().stream().map(GetSysUserRoleRelOutput::getRoleId).collect(Collectors.toList());
            String value = LoginConfig.adminRoles.findValue(context);
            session.setDevops(0);
            if (StringUtils.isNotBlank(value)) {
                String role = roleList.stream().filter(x -> value.contains(x)).findFirst().orElse(null);
                if (role != null) {
                    session.setDevops(1);
                }
            }
        }

        session.setIsSuperAdmin(UserConstant.ISSUPERADMIN_FALSE);
        session.setEnv(context.getEnv().getText());

        SystemUserRoleRelService roleRelService = context.getBean(SystemUserRoleRelService.class);
        GetSysUserRoleRelInput roleRelInput = new GetSysUserRoleRelInput();
        roleRelInput.setUserId(session.getId());
        XListResult<GetSysUserRoleRelOutput> roleRelResult = roleRelService.getSysRoleByUserId(context, roleRelInput);
        roleRelResult.throwIfFail();
        List<String> roleList = roleRelResult.getResult().stream().map(GetSysUserRoleRelOutput::getRoleId).collect(Collectors.toList());
        String value = LoginConfig.adminRoles.findValue(context);

        if (StringUtils.isNotBlank(value)) {
            String role = roleList.stream().filter(x -> value.contains(x)).findFirst().orElse(null);
            if (role != null) {
                session.setIsSuperAdmin(UserConstant.ISSUPERADMIN_TRUE);
            }
        }

        return XSingleResult.success(session);
    }


    // 查找所属机构
    private GetSysOrganizationViewOutput findOwnerOrg(XContext context, GetSysOrganizationViewOutput org) {
        if (OuLevelConstant.JI_GUAN_CHU_SHI.getCode().equals(org.getOuLevel()) ||
                OuLevelConstant.JI_GUAN_KE_SHI.getCode().equals(org.getOuLevel())) {
            //查找上级机构
            List<String> idPathList = Splitter.on('/').splitToList(org.getOuPath());
            if (idPathList.size() <= 1) {
                return null;
            } else {
                //查找上级机构
                String pareOuId = idPathList.get(idPathList.size() - 2);
                SysOrganizationCloudService organizationService = context.getBean(SysOrganizationCloudService.class);
                GetSysOrganizationViewInput input = new GetSysOrganizationViewInput();
                input.setId(pareOuId);
                XSingleResult<GetSysOrganizationViewOutput> sysOrganizationById = organizationService.getSysOrganizationById(context, input);
                if (sysOrganizationById.isSuccess()) {
                    return findOwnerOrg(context, sysOrganizationById.getResult());
                } else {
                    return null;
                }
            }
        } else {
            return org;
        }
    }
}
