package pps.core.system.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.CreateSysDictionaryInput;
import pps.cloud.system.service.data.QuerySysDictionaryViewInput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import pps.core.common.pattern.Builder;
import pps.core.system.cache.SysDictionaryCache;
import pps.core.system.entity.SysDictionaryEnt;
import pps.core.system.entity.SysOrganizationDictionaryEnt;
import pps.core.system.entity.SysOrganizationRelEnt;
import pps.core.system.mapper.SysDictionaryMapper;
import pps.core.system.mapper.SysOrganizationDictionaryMapper;
import pps.core.system.mapper.SysOrganizationRelMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;

import java.util.*;
import java.util.stream.Collectors;

@XService
public class SysDictionaryServiceImpl implements SystemDictionaryService {

    @Override
    public XListResult<QuerySysDictionaryViewOutput> querySysDictionarysByAlias(XContext context, CreateSysDictionaryInput input) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        SysOrganizationDictionaryMapper orgDicmapper = context.getBean(SysOrganizationDictionaryMapper.class);
        SysOrganizationRelMapper orgRelMapper = context.getBean(SysOrganizationRelMapper.class);
        List<SysDictionaryCache> list = SysDictionaryCache.list(context, input.getAlias());
        QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysDictionaryEnt::getAlias, input.getAlias());
        SysDictionaryEnt sysDictionaryEnt = mapper.selectOne(queryWrapper);
        list = list.stream().filter((SysDictionaryCache dic) -> StringUtils.equals(dic.getPid(), sysDictionaryEnt.getId()) && !StringUtils.equals(dic.getAlias(), input.getAlias()))
                .collect(Collectors.toList());
        if (ObjectUtils.isEmpty(list)) {
            list = new ArrayList<>();
            SysDictionaryCache.set(context, input.getAlias(), list);
            //查询alias字典
            QueryWrapper<SysDictionaryEnt> query = new QueryWrapper();
            query.lambda().eq(SysDictionaryEnt::getAlias, input.getAlias()).gt(SysDictionaryEnt::getEndTime, new Date());
            SysDictionaryEnt sysDictionary = mapper.selectOne(query);
            if (ObjectUtils.isNotEmpty(sysDictionary)) {
                SysDictionaryCache sysDictionaryCache = new SysDictionaryCache();
                XCopyUtils.copyObject(sysDictionary, sysDictionaryCache);
                list.add(sysDictionaryCache);
                //查询alias字典下级字典项
                QueryWrapper<SysDictionaryEnt> queryItems = new QueryWrapper();
                if (ObjectUtils.isNotEmpty(sysDictionary.getIsOrg()) && sysDictionary.getIsOrg() == 1) {
                    //查询当前登录人的组织机构层级路径
                    QueryWrapper<SysOrganizationRelEnt> queryOrgRel = new QueryWrapper<>();
                    queryOrgRel.lambda().eq(SysOrganizationRelEnt::getOuId, input.getOuId())
                            .gt(SysOrganizationRelEnt::getEndTime, new Date());
                    SysOrganizationRelEnt sysOrganizationRelEnt = orgRelMapper.selectOne(queryOrgRel);
                    //根据字典id查询
                    QueryWrapper<SysOrganizationDictionaryEnt> queryOrgDic = new QueryWrapper();
                    queryOrgDic.lambda().eq(SysOrganizationDictionaryEnt::getParentDicCode, sysDictionary.getId())
                            .gt(SysOrganizationDictionaryEnt::getEndTime, new Date());
                    List<SysOrganizationDictionaryEnt> sysOrganizationDictionaryEnts = orgDicmapper.selectList(queryOrgDic);
                    String ouIdPath = sysOrganizationRelEnt.getOuIdPath();
                    String ouId = sysOrganizationRelEnt.getOuId();
                    if (StringUtils.isNotBlank(ouIdPath)) {
                        List<String> dicIds = sysOrganizationDictionaryEnts.stream().map(p -> {
                            if (ouIdPath.indexOf(p.getOuId()) != -1 && p.getUseLowerLevel() == 1) {
                                return p.getDicCode();
                            } else if (ouId.indexOf(p.getOuId()) != -1) {
                                return p.getDicCode();
                            } else {
                                return "1";
                            }
                        }).collect(Collectors.toList());
                        queryItems.lambda().in(SysDictionaryEnt::getId, dicIds);
                    }
                }
                queryItems.lambda().eq(SysDictionaryEnt::getPid, sysDictionaryCache.getId()).gt(SysDictionaryEnt::getEndTime, new Date());
                List<SysDictionaryEnt> sysDictionaryList = mapper.selectList(queryItems);
                if (ObjectUtils.isNotEmpty(sysDictionaryList)) {
                    if (ObjectUtils.isNotEmpty(list)) {
                        for (SysDictionaryEnt ent : sysDictionaryList) {
                            SysDictionaryCache cache = new SysDictionaryCache();
                            XCopyUtils.copyObject(ent, cache);
                            list.add(cache);
                        }
                    }
                    Collections.sort(sysDictionaryList, new Comparator<SysDictionaryEnt>() {
                        @Override
                        public int compare(SysDictionaryEnt o1, SysDictionaryEnt o2) {
                            if (ObjectUtils.isNotEmpty(o1.getSort()) && ObjectUtils.isNotEmpty(o2.getSort())) {
                                return o1.getSort() - o2.getSort();
                            } else {
                                return 0;
                            }
                        }
                    });
                    List<QuerySysDictionaryViewOutput> outputs = XCopyUtils.copyNewList(sysDictionaryList, QuerySysDictionaryViewOutput.class);
                    return XListResult.success(outputs);
                }
            }
        }
        if (sysDictionaryEnt != null && ObjectUtils.isNotEmpty(sysDictionaryEnt.getIsOrg()) && sysDictionaryEnt.getIsOrg() == 1) {
            //查询当前登录人的组织机构层级路径
            QueryWrapper<SysOrganizationRelEnt> queryOrgRel = new QueryWrapper<>();
            queryOrgRel.lambda().eq(SysOrganizationRelEnt::getOuId, input.getOuId())
                    .gt(SysOrganizationRelEnt::getEndTime, new Date());
            SysOrganizationRelEnt sysOrganizationRelEnt = orgRelMapper.selectOne(queryOrgRel);
            //根据字典id查询
            QueryWrapper<SysOrganizationDictionaryEnt> queryOrgDic = new QueryWrapper();
            queryOrgDic.lambda().eq(SysOrganizationDictionaryEnt::getParentDicCode, sysDictionaryEnt.getId())
                    .gt(SysOrganizationDictionaryEnt::getEndTime, new Date());
            List<SysOrganizationDictionaryEnt> sysOrganizationDictionaryEnts = orgDicmapper.selectList(queryOrgDic);
            String ouIdPath = sysOrganizationRelEnt.getOuIdPath();
            String ouId = sysOrganizationRelEnt.getOuId();
            if (StringUtils.isNotBlank(ouIdPath)) {
                List<String> dicIds = sysOrganizationDictionaryEnts.stream().map(p -> {
                    if (ouIdPath.indexOf(p.getOuId()) != -1 && p.getUseLowerLevel() == 1) {
                        return p.getDicCode();
                    } else if (ouId.indexOf(p.getOuId()) != -1) {
                        return p.getDicCode();
                    } else {
                        return "1";
                    }
                }).collect(Collectors.toList());
                list = list.stream().filter(i -> dicIds.contains(i.getId())).collect(Collectors.toList());
            }
        }
        Collections.sort(list, (o1, o2) -> {
            if (ObjectUtils.isNotEmpty(o1.getSort()) && ObjectUtils.isNotEmpty(o2.getSort())) {
                return o1.getSort() - o2.getSort();
            } else {
                return 0;
            }
        });
        List<QuerySysDictionaryViewOutput> outputs = XCopyUtils.copyNewList(list, QuerySysDictionaryViewOutput.class);
        return XListResult.success(outputs);
    }

    @Override
    public XListResult<QuerySysDictionaryViewOutput> queryChildSysDictionarysByParentAlias(XContext context, QuerySysDictionaryViewInput input) {
        List<QuerySysDictionaryViewOutput> resultList = new ArrayList<>();
        SysDictionaryMapper sysDictionaryMapper = context.getBean(SysDictionaryMapper.class);
        List<SysDictionaryEnt> list = sysDictionaryMapper.selectList(Builder.create(QueryWrapper.class).build(queryWrapper -> {
            queryWrapper.eq("alias", input.getAlias());
        }));
        if (list.isEmpty()) {
            return XListResult.success(resultList);
        }
        SysDictionaryEnt parentDictionaryEnt = list.get(0);
        list = sysDictionaryMapper.selectList(Builder.create(QueryWrapper.class).build(queryWrapper -> {
            queryWrapper.eq("pid", parentDictionaryEnt.getId());
        }));
        resultList = XCopyUtils.copyNewList(list, QuerySysDictionaryViewOutput.class);
        return XListResult.success(resultList);
    }
}