package pps.core.system.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import pps.cloud.base.service.BaseBaseClientCloudService;
import pps.cloud.base.service.data.BaseNameInput;
import pps.cloud.base.service.data.FindBaseClientOutput;
import pps.cloud.base.service.data.UpdateBaseClientCloudInput;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.MD5Util;
import pps.core.system.entity.*;
import pps.core.system.mapper.*;
import pps.core.system.service.data.sys_out_user_create_log.CreateSysOutUserCreateLogViewInput;
import pps.core.system.service.data.sys_out_user_create_log.QuerySysOutUserCreateLogViewInput;
import pps.core.system.service.data.sys_out_user_create_log.QuerySysOutUserCreateLogViewOutput;
import pps.core.system.utils.UserInfoUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XDateUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;

@XText("外部用户创建管理")
@XService
public class SysOutUserCreateLogService {

    @XText("查询外部用户创建列表")
    @XApiGet
    public XPageResult<QuerySysOutUserCreateLogViewOutput> querySysOutUserCreateLogViewPage(XContext context, QuerySysOutUserCreateLogViewInput input) {
        SysOutUserCreateLogMapper mapper = context.getBean(SysOutUserCreateLogMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<SysOutUserCreateLogEnt> queryWrapper = new QueryWrapper<>();
        if (input.getStatus() != null) {
            queryWrapper.lambda().eq(SysOutUserCreateLogEnt::getStatus, input.getStatus());
        }
        if (StringUtils.isNotBlank(input.getOuId())) {
            queryWrapper.lambda().eq(SysOutUserCreateLogEnt::getOuId, input.getOuId());
        }
        if (StringUtils.isNotBlank(input.getClientName())) {
            queryWrapper.lambda().like(SysOutUserCreateLogEnt::getClientName, input.getClientName());
        }
        if (StringUtils.isNotBlank(input.getLoginName())) {
            queryWrapper.lambda().like(SysOutUserCreateLogEnt::getLoginName, input.getLoginName());
        }
        if (StringUtils.isNotBlank(input.getUserName())) {
            queryWrapper.lambda().like(SysOutUserCreateLogEnt::getUserName, input.getUserName());
        }
        if (StringUtils.isNotBlank(input.getStartDataDate())) {
            queryWrapper.lambda().ge(SysOutUserCreateLogEnt::getCreateTime, input.getStartDataDate());
        }
        if (StringUtils.isNotBlank(input.getEndDataDate())) {
            queryWrapper.lambda().le(SysOutUserCreateLogEnt::getCreateTime, input.getEndDataDate());
        }
        queryWrapper.lambda().orderByDesc(SysOutUserCreateLogEnt::getCreateTime);
        XPageResult<QuerySysOutUserCreateLogViewOutput> query = XMapperHelper.query(mapper, input, queryWrapper, QuerySysOutUserCreateLogViewOutput.class);
        List<QuerySysOutUserCreateLogViewOutput> result = query.getResult().getItems();
        result.stream().forEach(item -> {
            if (XStringUtils.isNotEmpty(item.getOuLevel())) {
                switch (item.getOuLevel()) {
                    case "0":
                        item.setOuLevelName("托运商");
                        break;
                    case "1":
                        item.setOuLevelName("上游用户");
                        break;
                    case "2":
                        item.setOuLevelName("下游用户");
                        break;
                }
            }
            if (XStringUtils.isNotEmpty(item.getMediumList())) {
                String[] mediumList = item.getMediumList().split(",");
                List<String> mediumNameList = new ArrayList<>();
                for (String medium : mediumList) {
                    switch (medium) {
                        case "GAS":
                            mediumNameList.add("天然气");
                            break;
                        case "LNG":
                            mediumNameList.add("LNG");
                            break;
                        case "OIL":
                            mediumNameList.add("原油");
                            break;
                        case "CPY":
                            mediumNameList.add("成品油");
                            break;
                    }
                }
                item.setMediumListName(String.join(",", mediumNameList));
            }
        });
        return query;
    }

    @XText("审批外部用户")
    @XApiPost
    public XServiceResult approveOutUser(XContext context, CreateSysOutUserCreateLogViewInput input) {
        return XTransactionHelper.begin(context, () -> {
            SysOutUserCreateLogMapper mapper = context.getBean(SysOutUserCreateLogMapper.class);
            QueryWrapper<SysOutUserCreateLogEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysOutUserCreateLogEnt::getId, input.getId());
            SysOutUserCreateLogEnt ent = mapper.selectOne(queryWrapper);
            if (null == ent || ent.getStatus() == 1 || ent.getStatus() == 2)
                return XServiceResult.error(500, "数据已被处理");
            PpsUserSession session = context.getSession(PpsUserSession.class);
            //1.判断客户信息是否存在，如果存在，进行下一步，如果不存在，直接返回
            Date date = new Date();
            ent.setModifyById(session.getId());
            ent.setModifyByName(session.getUserName());
            ent.setModifyTime(date);
            ent.setStatus(input.getStatus());
            if (input.getStatus() == 1) {
                String ouLevel = "SUPPLIER";
                BaseBaseClientCloudService cloudService = context.getBean(BaseBaseClientCloudService.class);
                BaseNameInput baseNameInput = new BaseNameInput();
                baseNameInput.setName(ent.getClientName());
                FindBaseClientOutput client = null;
                switch (ent.getOuLevel()) {
                    case "0":
                        XSingleResult<FindBaseClientOutput> resultShipper = cloudService.selectShipperByName(context, baseNameInput);
                        resultShipper.throwIfFail();
                        client = resultShipper.getResult();
                        break;
                    case "1":
                        XSingleResult<FindBaseClientOutput> resultUpClient = cloudService.selectUpClientByName(context, baseNameInput);
                        resultUpClient.throwIfFail();
                        client = resultUpClient.getResult();
                        ouLevel = "UP_CLIENT";
                        break;
                    case "2":
                        XSingleResult<FindBaseClientOutput> resultDownClient = cloudService.selectDownClientByName(context, baseNameInput);
                        resultDownClient.throwIfFail();
                        client = resultDownClient.getResult();
                        ouLevel = "DOWN_CLIENT";
                        break;
                }
                if (null != client && XStringUtils.isEmpty(client.getOuId())) {
                    //2.判断客户是否存在组织机构，如果不存在，新增组织机构，并与客户做业务绑定
                    //2.1创建组织机构
                    SysOrganizationEnt organization = new SysOrganizationEnt();
                    SysOrganizationMapper organizationMapper = context.getBean(SysOrganizationMapper.class);
                    organization.setId(UUID.randomUUID().toString());
                    organization.setOuName(ent.getClientName());
                    organization.setShortName(ent.getClientName());
                    organization.setOuLevel(ouLevel);
                    organization.setCreateById(session.getId());
                    organization.setCreateByName(session.getUserName());
                    organization.setCreateTime(date);
                    organization.setIsDeleted(1);
                    organizationMapper.insert(organization);
                    //2.1.1赋予组织机构介质类型
                    SysOrganizationPropertyMapper propertyMapper = context.getBean(SysOrganizationPropertyMapper.class);
                    String[] mediumList = ent.getMediumList().split(",");
                    for (String item : mediumList) {
                        SysOrganizationPropertyEnt organizationPropertyEnt = new SysOrganizationPropertyEnt();
                        organizationPropertyEnt.setId(UUID.randomUUID().toString());
                        organizationPropertyEnt.setOuId(organization.getId());
                        organizationPropertyEnt.setPropertyCode("MEDIUM_TYPE");
                        organizationPropertyEnt.setPropertyValue(item);
                        organizationPropertyEnt.setEffectTime(date);
                        organizationPropertyEnt.setEndTime(XDateUtils.parse("2099-12-31 00:00:00"));
                        organizationPropertyEnt.setCreateById(session.getId());
                        organizationPropertyEnt.setCreateByName(session.getUserName());
                        organizationPropertyEnt.setCreateTime(date);
                        propertyMapper.insert(organizationPropertyEnt);
                    }
                    //2.2组织机构挂在外部企业下面
                    SysOrganizationRelMapper organizationRelMapper = context.getBean(SysOrganizationRelMapper.class);
                    SysOrganizationRelEnt organizationRelEnt = new SysOrganizationRelEnt();
                    organizationRelEnt.setId(UUID.randomUUID().toString());
                    organizationRelEnt.setOuId(organization.getId());
                    organizationRelEnt.setParentOuId("0cfa0232-2ed8-4721-a8f3-5febf9122510");
                    organizationRelEnt.setOuIdPath("F6AC16F3-C184-4836-BE87-65AA5F1695B8/0cfa0232-2ed8-4721-a8f3-5febf9122510/" + organizationRelEnt.getId());
                    organizationRelEnt.setEffectTime(date);
                    organizationRelEnt.setEndTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse("9999-12-31 00:00:00"));
                    organizationRelEnt.setCreateById(session.getId());
                    organizationRelEnt.setCreateByName(session.getUserName());
                    organizationRelEnt.setCreateTime(date);
                    organizationRelEnt.setSort(999);
                    organizationRelMapper.insert(organizationRelEnt);
                    //2.3更新客户组织机构id
                    client.setOuId(organization.getId());
                    BaseBaseClientCloudService clientCloudService = context.getBean(BaseBaseClientCloudService.class);
                    UpdateBaseClientCloudInput clientCloudInput = XCopyUtils.copyNewObject(client, UpdateBaseClientCloudInput.class);
                    clientCloudService.updateBaseClient(context, clientCloudInput);
                    ent.setOuId(organization.getId());
                    ent.setOuName(organization.getOuName());
                    ent.setClientId(client.getId());
                    ent.setClientName(client.getClientName());
                }
                //3新增用户
                SysUserMapper userMapper = context.getBean(SysUserMapper.class);
                SysUserEnt userEnt = XCopyUtils.copyNewObject(ent, SysUserEnt.class);
                userEnt.setId(UUID.randomUUID().toString());
                userEnt.setCreateById(session.getId());
                userEnt.setCreateByName(session.getUserName());
                userEnt.setCreateTime(date);
                userEnt.setIsOuter(1);
                userEnt.setIsInner(2);
                if (XStringUtils.isNotEmpty(userEnt.getPassword()))
                    userEnt.setPassword(MD5Util.encryptPassword(userEnt.getPassword()));
                else
                    userEnt.setPassword("2e7832f9c3a0ca243c559017c07cceed");
                userEnt.setPhone(UserInfoUtil.encrypt(userEnt.getPhone()));
                userMapper.insert(userEnt);
                //插入用户组织机构关系
                SysUserOrganizationRelEnt sysUserOrganizationRelEnt = new SysUserOrganizationRelEnt();
                sysUserOrganizationRelEnt.setId(UUID.randomUUID().toString());
                sysUserOrganizationRelEnt.setOuId(client.getOuId());
                sysUserOrganizationRelEnt.setUserId(userEnt.getId());
                sysUserOrganizationRelEnt.setCreateById(session.getId());
                sysUserOrganizationRelEnt.setCreateByName(session.getUserName());
                sysUserOrganizationRelEnt.setCreateTime(date);
                sysUserOrganizationRelEnt.setEffectTime(new Date());
                sysUserOrganizationRelEnt.setEndTime(XDateUtils.tryParse("2099-12-31"));
                SysUserOrganizationRelMapper userOrganizationRelMapper = context.getBean(SysUserOrganizationRelMapper.class);
                userOrganizationRelMapper.insert(sysUserOrganizationRelEnt);
                //插入用户角色关系
                SysUserRoleRelMapper userRoleRelMapper = context.getBean(SysUserRoleRelMapper.class);
                QueryWrapper<SysUserRoleRelEnt> queryRelWrapper = new QueryWrapper<>();
                queryRelWrapper.lambda().eq(SysUserRoleRelEnt::getUserId, "abd73e81-2b0a-4585-a71b-b52c7a36a2cb");
                List<SysUserRoleRelEnt> userRoleRelEnts = userRoleRelMapper.selectList(queryRelWrapper);
                if (!CollectionUtils.isEmpty(userRoleRelEnts)) {
                    userRoleRelEnts.stream().forEach(item -> {
                        item.setId(UUID.randomUUID().toString());
                        item.setUserId(userEnt.getId());
                        userRoleRelMapper.insert(item);
                    });
                }
            }
            mapper.updateById(ent);
            return XServiceResult.OK;
        });
    }

    @XText("创建外部用户")
    @XApiPost
    @XApiAnonymous
    public XServiceResult createOutUser(XContext context, CreateSysOutUserCreateLogViewInput input) {
        return XTransactionHelper.begin(context, () -> {
            SysOutUserCreateLogMapper mapper = context.getBean(SysOutUserCreateLogMapper.class);
            Date date = new Date();
            //1.判断客户信息是否存在，如果存在，进行下一步，如果不存在，直接返回
            SysOutUserCreateLogEnt ent = XCopyUtils.copyNewObject(input, SysOutUserCreateLogEnt.class);
            ent.setCreateTime(date);
            ent.setModifyTime(date);
            ent.setStatus(0);
            Boolean flag = true;
            StringBuffer buffer = new StringBuffer();
            if (XStringUtils.isEmpty(ent.getClientName())) {
                flag = false;
                buffer.append("客户名称不能为空");
            }
            if (XStringUtils.isEmpty(ent.getLoginName())) {
                flag = false;
                buffer.append("登录名不能为空");
            }
            if (XStringUtils.isEmpty(ent.getUserName())) {
                flag = false;
                buffer.append("用户姓名不能为空");
            }
//            if(XStringUtils.isEmpty(ent.getPassword())){
//                flag = false;
//                buffer.append("登录密码不能为空");
//            }
//            if(XStringUtils.isEmpty(ent.getIamLoginName())){
//                flag = false;
//                buffer.append("IAM接口登录名不能为空");
//            }
//            if(XStringUtils.isEmpty(ent.getUserCode())){
//                flag = false;
//                buffer.append("员工编码不能为空");
//            }
//            if(XStringUtils.isEmpty(ent.getDuty())){
//                flag = false;
//                buffer.append("职务不能为空");
//            }
//            if(XStringUtils.isEmpty(ent.getPostalCode())){
//                flag = false;
//                buffer.append("邮编不能为空");
//            }
//            if(XStringUtils.isEmpty(ent.getPhone())){
//                flag = false;
//                buffer.append("手机号不能为空");
//            }
//            if(XStringUtils.isEmpty(ent.getEmail())){
//                flag = false;
//                buffer.append("用户邮箱不能为空");
//            }
//            if(XStringUtils.isEmpty(ent.getAddress())){
//                flag = false;
//                buffer.append("通讯地址不能为空");
//            }
            if (flag) {
                //2根据日志，判断用户操作次数
                SysOutUserSearchCilentLogMapper searchCilentLogMapper = context.getBean(SysOutUserSearchCilentLogMapper.class);
                QueryWrapper<SysOutUserSearchClientLogEnt> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda().eq(SysOutUserSearchClientLogEnt::getIpAddress, context.getWebRequest().getClientIp());
                queryWrapper.lambda().ge(SysOutUserSearchClientLogEnt::getCreateTime, XDateUtils.addMunites(new Date(), -5));
                Long count = searchCilentLogMapper.selectCount(queryWrapper);
                if (count > 5)
                    return XServiceResult.error(500, "客户查询失败超过5次");
                BaseBaseClientCloudService cloudService = context.getBean(BaseBaseClientCloudService.class);
                BaseNameInput baseNameInput = new BaseNameInput();
                baseNameInput.setName(ent.getClientName());
                FindBaseClientOutput client = null;
                switch (input.getOuLevel()) {
                    case "0":
                        XSingleResult<FindBaseClientOutput> resultShipper = cloudService.selectShipperByName(context, baseNameInput);
                        resultShipper.throwIfFail();
                        client = resultShipper.getResult();
                        break;
                    case "1":
                        XSingleResult<FindBaseClientOutput> resultUpClient = cloudService.selectUpClientByName(context, baseNameInput);
                        resultUpClient.throwIfFail();
                        client = resultUpClient.getResult();
                        break;
                    case "2":
                        XSingleResult<FindBaseClientOutput> resultDownClient = cloudService.selectDownClientByName(context, baseNameInput);
                        resultDownClient.throwIfFail();
                        client = resultDownClient.getResult();
                        break;
                }
                if (null != client) {
                    //3如果客户存在，新增用户
                    //3.1判断账户、iam账户是否重复
                    SysUserMapper userMapper = context.getBean(SysUserMapper.class);
                    QueryWrapper<SysUserEnt> entQueryWrapper = new QueryWrapper<>();
                    entQueryWrapper.lambda().eq(SysUserEnt::getLoginName, ent.getLoginName());
                    Long loginCount = userMapper.selectCount(entQueryWrapper);
                    QueryWrapper<SysOutUserCreateLogEnt> logEntQueryWrapper = new QueryWrapper<>();
                    logEntQueryWrapper.lambda().eq(SysOutUserCreateLogEnt::getLoginName, ent.getLoginName());
                    Long loginLogCount = mapper.selectCount(logEntQueryWrapper);
                    if (loginCount > 0 || loginLogCount > 0) {
                        return XServiceResult.error(500, "登录账户已存在");
                    }
                    if (XStringUtils.isNotEmpty(ent.getIamLoginName())) {
                        entQueryWrapper.clear();
                        entQueryWrapper.lambda().eq(SysUserEnt::getIamLoginName, ent.getIamLoginName());
                        Long loginIamCount = userMapper.selectCount(entQueryWrapper);
                        logEntQueryWrapper.clear();
                        logEntQueryWrapper.lambda().eq(SysOutUserCreateLogEnt::getIamLoginName, ent.getIamLoginName());
                        Long loginLogiamCount = mapper.selectCount(logEntQueryWrapper);
                        if (loginIamCount > 0 || loginLogiamCount > 0) {
                            return XServiceResult.error(500, "IAM登录账户已存在");
                        }
                    }
                    if (XStringUtils.isNotEmpty(ent.getPhone())) {
                        entQueryWrapper.clear();
                        entQueryWrapper.lambda().eq(SysUserEnt::getPhone, ent.getPhone());
                        Long loginPhoneCount = userMapper.selectCount(entQueryWrapper);
                        logEntQueryWrapper.clear();
                        logEntQueryWrapper.lambda().eq(SysOutUserCreateLogEnt::getPhone, ent.getPhone());
                        Long loginLogPhoneCount = mapper.selectCount(logEntQueryWrapper);
                        if (loginPhoneCount > 0 || loginLogPhoneCount > 0) {
                            return XServiceResult.error(500, "手机号码已存在");
                        }
                    }
                    if (!CollectionUtils.isEmpty(input.getMediumList())) {
                        List<String> mediumList = input.getMediumList();
                        String str = "";
                        for (String item : mediumList) {
                            str += item + ",";
                        }
                        ent.setMediumList(str.substring(0, str.length() - 1));
                    }
                    // ent.setMediumList(StringUtils.join("," ,input.getMediumList()));
                    ent.setClientId(client.getId());
                    if (XStringUtils.isNotEmpty(client.getOuId())) {
                        SysOrganizationMapper orgMapper = context.getBean(SysOrganizationMapper.class);
                        QueryWrapper<SysOrganizationEnt> orgQueryWrapper = new QueryWrapper<>();
                        orgQueryWrapper.lambda().eq(SysOrganizationEnt::getId, client.getOuId());
                        SysOrganizationEnt sysOrganizationEnt = orgMapper.selectOne(orgQueryWrapper);
                        if (null != sysOrganizationEnt) {
                            ent.setOuId(sysOrganizationEnt.getId());
                            ent.setOuName(sysOrganizationEnt.getOuName());
                        }
                    }
                    mapper.insert(ent);
                } else {
                    //根据ip记录日志，防止用户重复操作。
                    SysOutUserSearchClientLogEnt logEnt = new SysOutUserSearchClientLogEnt();
                    logEnt.setId(UUID.randomUUID().toString());
                    logEnt.setErrorMsg(ent.getClientName());
                    logEnt.setIpAddress(context.getWebRequest().getClientIp());
                    logEnt.setCreateTime(new Date());
                    searchCilentLogMapper.insert(logEnt);
                    return XServiceResult.error(500, "客户不存在");
                }
            } else {
                return XServiceResult.error(500, buffer.toString());
            }
            return XServiceResult.OK;
        });
    }
}