package pps.core.system.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import pps.core.common.session.PpsUserSession;
import pps.core.system.entity.SysSmsGroupEnt;
import pps.core.system.entity.SysSmsGroupUserEnt;
import pps.core.system.entity.SysSmsGroupUserView;
import pps.core.system.entity.SysSmsGroupView;
import pps.core.system.mapper.SysSmsGroupMapper;
import pps.core.system.mapper.SysSmsGroupUserMapper;
import pps.core.system.mapper.SysSmsGroupUserViewMapper;
import pps.core.system.mapper.SysSmsGroupViewMapper;
import pps.core.system.service.data.sys_sms_group.CreateSysSmsGroupInput;
import pps.core.system.service.data.sys_sms_group.CreateSysSmsGroupOutput;
import pps.core.system.service.data.sys_sms_group.QuerySysSmsGroupInput;
import pps.core.system.service.data.sys_sms_group.QuerySysSmsGroupOutput;
import pps.core.system.service.data.sys_sms_group_user.CreateSysSmsGroupUserInput;
import pps.core.system.utils.UserInfoUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;

import java.util.*;

@XText("收信组管理")
@XService
public class SysSmsGroupService {
    @XText("添加收信组")
    @XApiAnonymous
    @XApiPost
    public XServiceResult createSysSmsGroup(XContext context, CreateSysSmsGroupInput input) {
        SysSmsGroupMapper mapper = context.getBean(SysSmsGroupMapper.class);
        SysSmsGroupUserMapper userMapper = context.getBean(SysSmsGroupUserMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        SysSmsGroupEnt entity;
        SysSmsGroupUserEnt userEnt;
        QueryWrapper<SysSmsGroupEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysSmsGroupEnt::getSmsGroupId, input.getSmsGroupId());
        entity = mapper.selectOne(queryWrapper);
        Date date = new Date();
        if (entity != null) {
            this.deleteSysSmsGroup(context, input);
            entity.setModifyTime(date);
            entity.setSmsGroupName(input.getSmsGroupName());
            entity.setIsShar(input.getIsShar());
            if (session != null) {
                entity.setModifyById(session.getId());
                entity.setModifyById(session.getUserName());
                entity.setOuId(session.getOuId());
                entity.setOuName(session.getOuName());
            }
        } else {
            entity = new SysSmsGroupEnt();
            XCopyUtils.copyObject(input, entity);
            String uuid = UUID.randomUUID().toString();
            entity.setSmsGroupId(uuid);
            entity.setCreateTime(date);
            entity.setIsDeleted("0");
            if (session != null) {
                entity.setCreateById(session.getId());
                entity.setCreateByName(session.getUserName());
                entity.setOuId(session.getOuId());
                entity.setOuName(session.getOuName());
            }
        }
        if (input.getGroupUserInputList().size() > 0) {
            try {
                for (CreateSysSmsGroupUserInput item : input.getGroupUserInputList()) {
                    String groupUserId = UUID.randomUUID().toString();
                    userEnt = new SysSmsGroupUserEnt();
                    XCopyUtils.copyObject(item, userEnt);
                    userEnt.setMobile(UserInfoUtil.encrypt(item.getMobile()));
                    userEnt.setSmsGroupUserId(groupUserId);
                    userEnt.setIsDeleted("0");
                    userEnt.setSmsGroupId(entity.getSmsGroupId());
                    userMapper.insert(userEnt);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("查询收信组列表")
    @XApiAnonymous
    @XApiPost
    public XPageResult<QuerySysSmsGroupOutput> querySysSmsGroup(XContext context, QuerySysSmsGroupInput input) {
        SysSmsGroupViewMapper mapper = context.getBean(SysSmsGroupViewMapper.class);
        SysSmsGroupView record = new SysSmsGroupView();
        List<SysSmsGroupView> list;
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        PpsUserSession session = context.getSession(PpsUserSession.class);
        if (session != null) {
            record.setCreateById(session.getId());
            record.setOuId(session.getOuId());
        }
        list = mapper.selectGroupList(record);
        PageInfo<SysSmsGroupView> pageInfo = new PageInfo<>(list);
        List<QuerySysSmsGroupOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysSmsGroupOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("根据查询自己和共享")
    @XApiAnonymous
    @XApiPost
    public XPageResult<QuerySysSmsGroupOutput> querySysSmsGroupList(XContext context, QuerySysSmsGroupInput input) {
        SysSmsGroupViewMapper mapper = context.getBean(SysSmsGroupViewMapper.class);
        SysSmsGroupUserViewMapper usermapper = context.getBean(SysSmsGroupUserViewMapper.class);
        SysSmsGroupView record = new SysSmsGroupView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        PpsUserSession session = context.getSession(PpsUserSession.class);
        if (session != null) {
            record.setCreateById(session.getId());
            record.setOuId(session.getOuId());
        }
        List<SysSmsGroupView> listGroup = mapper.selectList(record);
        if (listGroup.size() > 0) {
            listGroup.forEach(item -> {
                SysSmsGroupUserView userrecord = new SysSmsGroupUserView();
                userrecord.setSmsGroupId(item.getSmsGroupId());
                List<SysSmsGroupUserView> listUser = usermapper.selectSysSmsGroupUserList(userrecord);
                StringBuffer stringUserBuffer = new StringBuffer();
                StringBuffer stringPhoneBuffer = new StringBuffer();
                if (listUser.size() > 0) {
                    listUser.forEach(user -> {
                        if (!StringUtils.isNotBlank(stringUserBuffer)) {
                            stringUserBuffer.append(user.getSmsUsername());
                            stringPhoneBuffer.append(UserInfoUtil.decrypt(user.getMobile()));
                        } else {
                            stringUserBuffer.append("," + user.getSmsUsername());
                            stringPhoneBuffer.append("," + UserInfoUtil.decrypt(user.getMobile()));
                        }
                    });
                    item.setSmsUsername(stringUserBuffer.toString());
                    item.setMobile(stringPhoneBuffer.toString());
                }
            });
        }
        PageInfo<SysSmsGroupView> pageInfo = new PageInfo<>(listGroup);
        List<QuerySysSmsGroupOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysSmsGroupOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());

    }

    @XText("根据id查询列表")
    @XApiAnonymous
    @XApiPost
    public XSingleResult<CreateSysSmsGroupOutput> querySysSmsGroupView(XContext context, CreateSysSmsGroupInput input) {
        SysSmsGroupViewMapper mapper = context.getBean(SysSmsGroupViewMapper.class);
        SysSmsGroupView record = new SysSmsGroupView();
        XCopyUtils.copyObject(input, record);
        SysSmsGroupView view = mapper.selectOne(record);
        CreateSysSmsGroupOutput output = new CreateSysSmsGroupOutput();
        XCopyUtils.copyObject(view, output);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        SysSmsGroupUserViewMapper usermapper = context.getBean(SysSmsGroupUserViewMapper.class);
        SysSmsGroupUserView userrecord = new SysSmsGroupUserView();
        XCopyUtils.copyObject(input, userrecord);
        List<SysSmsGroupUserView> list = usermapper.selectSysSmsGroupUserList(userrecord);
        if (list.size() > 0) {
            list.forEach(item -> {
                item.setMobile(UserInfoUtil.decrypt(item.getMobile()));
            });
            output.setGroupUserInputList(list);
        }
        return XSingleResult.success(output);
    }

    @XText("修改收信组")
    @XApiAnonymous
    @XApiPost
    public XServiceResult updateSysSmsGroup(XContext context, CreateSysSmsGroupInput input) {
        SysSmsGroupMapper mapper = context.getBean(SysSmsGroupMapper.class);
        QueryWrapper<SysSmsGroupEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysSmsGroupEnt::getSmsGroupId, input.getSmsGroupId());
        SysSmsGroupEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        PpsUserSession session = context.getSession(PpsUserSession.class);
        if (session != null) {
            Date date = new Date();
            entity.setModifyTime(date);
            entity.setModifyById(session.getId());
            entity.setModifyByName(session.getUserName());
            entity.setIsHide(input.getIsHide());
        }
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除收信组")
    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteSysSmsGroup(XContext context, CreateSysSmsGroupInput input) {
        SysSmsGroupMapper mapper = context.getBean(SysSmsGroupMapper.class);
        SysSmsGroupUserMapper userMapper = context.getBean(SysSmsGroupUserMapper.class);
        QueryWrapper<SysSmsGroupEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysSmsGroupEnt::getSmsGroupId, input.getSmsGroupId());
        QueryWrapper<SysSmsGroupUserEnt> queryUserWrapper = new QueryWrapper<>();
        queryUserWrapper.lambda().eq(SysSmsGroupUserEnt::getSmsGroupId, input.getSmsGroupId());
        SysSmsGroupEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        Map<String, Object> map = new HashMap<>();
        map.put("sms_group_id", input.getSmsGroupId());
        userMapper.deleteByMap(map);
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XText("获取收信组")
    @XApiAnonymous
    @XApiGet
    public XSingleResult<CreateSysSmsGroupOutput> getSysSmsGroup(XContext context, CreateSysSmsGroupInput input) {
        SysSmsGroupMapper mapper = context.getBean(SysSmsGroupMapper.class);
        QueryWrapper<SysSmsGroupEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysSmsGroupEnt::getSmsGroupId, input.getSmsGroupId());
        SysSmsGroupEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        CreateSysSmsGroupOutput output = new CreateSysSmsGroupOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }
}