package pps.core.system.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.util.Strings;
import org.springframework.util.CollectionUtils;
import pps.core.common.constant.UserConstant;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.RSAUtil;
import pps.core.system.config.LoginConfig;
import pps.core.system.constant.SysOrganizationCode;
import pps.core.system.entity.*;
import pps.core.system.error.LoginError;
import pps.core.system.mapper.*;
import pps.core.system.service.data.sys_user.*;
import pps.core.system.utils.UserInfoUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.XUser;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XEncryptUtils;
import xstartup.base.util.XJsonUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@XText("用户管理")
@XService
public class SysUserService {

    @XText("查询用户列表")
    @XApiPost
    public XPageResult<QuerySysUserViewOutput> selectSysUserViewPage(XContext context, QuerySysUserViewInput input) {
        SysUserViewMapper mapper = context.getBean(SysUserViewMapper.class);
        SysUserView record = new SysUserView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysUserView> list;
        record.setEndTime(new Date());
        PpsUserSession session = context.getSession(PpsUserSession.class);
        if (UserConstant.ISSUPERADMIN_FALSE.equals(session.getIsSuperAdmin())) {
            if (XStringUtils.isEmpty(input.getOuId())) {
                record.setOuId(session.getOuId());
            }
        }
        if (input.getHasOuId() != null && input.getHasOuId() == 0) {
            record.setHasOuId(input.getHasOuId() + "");
            list = mapper.selectSysUserListByNoOuId(record);
        } else
            list = mapper.selectSysUserListByOuId(record);
        PageInfo<SysUserView> pageInfo = new PageInfo<>(list);
        List<QuerySysUserViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysUserViewOutput.class);
        for (QuerySysUserViewOutput output : outputs) {
            output.setPhone(UserInfoUtil.desensitization(output.getPhone()));
            output.setIdentifyNo(UserInfoUtil.desensitization(output.getIdentifyNo()));
            output.setEmail(UserInfoUtil.desensitization(output.getEmail()));
            if (output.getStatus() == null)
                output.setStatus(0);
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("新增用户")
    @XApiPost
    public XServiceResult createSysUser(XContext context, CreateSysUserInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysUserMapper mapper = context.getBean(SysUserMapper.class);
            //查询是否存在相同的登录名，如果存在，提示错误信息
            QueryWrapper<SysUserEnt> isExitQueryWrapper = new QueryWrapper<>();
            isExitQueryWrapper.lambda().eq(SysUserEnt::getLoginName, input.getLoginName());
            List<SysUserEnt> isExit = mapper.selectList(isExitQueryWrapper);
            if (!CollectionUtils.isEmpty(isExit)) {
                return XServiceResult.error(993, "登录账户已存在！");
            }
            //查询是否存在有相同的Iam登录名，如果存在，提示错误信息
            if (XStringUtils.isNotEmpty(input.getIamLoginName())) {
                isExitQueryWrapper.clear();
                isExitQueryWrapper.lambda().eq(SysUserEnt::getIamLoginName, input.getIamLoginName());
                List<SysUserEnt> isIamExit = mapper.selectList(isExitQueryWrapper);
                if (!CollectionUtils.isEmpty(isIamExit)) {
                    return XServiceResult.error(993, "Iam登录账户已存在！");
                }
            }
            SysUserEnt entity = new SysUserEnt();
            XCopyUtils.copyObject(input, entity);
            if (XStringUtils.isEmpty(input.getPassword())) {
                return XServiceResult.error(993, "请输入密码！");
            }
            //String password = MD5Util.encryptPassword(AESUtil.decrypt(input.getPassword()));
            entity.setPassword(input.getPassword());
            String uuid = UUID.randomUUID().toString();
            entity.setId(uuid);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (session != null) {
                entity.setCreateById(session.getId());
                entity.setCreateByName(session.getUserName());
            }
            entity.setCreateTime(new Date());
            //加密敏感信息
            entity.setPhone(UserInfoUtil.encrypt(entity.getPhone()));
            entity.setIdentifyNo(UserInfoUtil.encrypt(entity.getIdentifyNo()));
            entity.setEmail(UserInfoUtil.encrypt(entity.getEmail()));
            mapper.insert(entity);
            //新增用户和组织机构关系表
            insertSysUserOrganizationRel(context, uuid, input.getOuId(), session);
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    //新增用户和组织机构关系表
    public XServiceResult insertSysUserOrganizationRel(XContext context, String uuid, String ouId, PpsUserSession session) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysUserOrganizationRelMapper sorMapper = context.getBean(SysUserOrganizationRelMapper.class);
            SysUserOrganizationRelEnt relEnt = new SysUserOrganizationRelEnt();
            relEnt.setId(UUID.randomUUID().toString());
            relEnt.setUserId(uuid);
            relEnt.setOuId(ouId);
            Date date = new Date();
            relEnt.setEffectTime(date);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            String datetime = "9999-12-31";
            try {
                relEnt.setEndTime(sdf.parse(datetime));
            } catch (ParseException e) {
                e.printStackTrace();
            }
            if (session != null) {
                relEnt.setCreateById(session.getId());
                relEnt.setCreateByName(session.getUserName());
            }
            relEnt.setCreateTime(date);
            sorMapper.insert(relEnt);
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("修改用户信息")
    @XApiPost
    public XServiceResult updateSysUser(XContext context, UpdateSysUserInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysUserMapper mapper = context.getBean(SysUserMapper.class);
            QueryWrapper<SysUserEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysUserEnt::getId, input.getId());
            SysUserEnt entity = mapper.selectOne(queryWrapper);
            if (entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            //查询是否存在相同的登录名，如果存在，提示错误信息
            queryWrapper.clear();
            queryWrapper.lambda().eq(SysUserEnt::getLoginName, input.getLoginName());
            List<SysUserEnt> isExit = mapper.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(isExit) && (isExit.size() > 1
                    || (isExit.size() == 1 && !isExit.get(0).getId().equals(input.getId())))) {
                return XServiceResult.error(993, "登录账户已存在！");
            }
            //查询是否存在有相同的Iam登录名，如果存在，提示错误信息
            if (XStringUtils.isNotEmpty(input.getIamLoginName())) {
                queryWrapper.clear();
                queryWrapper.lambda().eq(SysUserEnt::getIamLoginName, input.getIamLoginName());
                List<SysUserEnt> isIamExit = mapper.selectList(queryWrapper);
                if (!CollectionUtils.isEmpty(isIamExit) && (isIamExit.size() > 1
                        || (isIamExit.size() == 1 && !isIamExit.get(0).getId().equals(input.getId())))) {
                    return XServiceResult.error(993, "Iam登录账户已存在！");
                }
            }
            XCopyUtils.copyObject(input, entity);
            if (StringUtils.isNotBlank(entity.getPassword())) {
                //String password = MD5Util.encryptPassword(AESUtil.decrypt(input.getPassword()));
                entity.setPassword(input.getPassword());
            }
            entity.setModifyTime(new Date());
            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (session != null) {
                entity.setModifyById(session.getId());
                entity.setModifyByName(session.getUserName());
            }
            //加密敏感信息
            if (StringUtils.isNotBlank(input.getPhone()) && !input.getPhone().contains("*")) {
                entity.setPhone(UserInfoUtil.encrypt(input.getPhone()));
            } else {
                entity.setPhone(null);
            }
            if (StringUtils.isNotBlank(input.getIdentifyNo()) && !input.getIdentifyNo().contains("*")) {
                entity.setIdentifyNo(UserInfoUtil.encrypt(input.getIdentifyNo()));
            } else {
                entity.setIdentifyNo(null);
            }
            if (StringUtils.isNotBlank(input.getEmail()) && !input.getEmail().contains("*")) {
                entity.setEmail(UserInfoUtil.encrypt(input.getEmail()));
            } else {
                entity.setEmail(null);
            }
            mapper.updateById(entity);
            //修改用户和组织机构关系为失效
            updateSysUserOrganizationRel(context, input.getId(), input.getOuId(), session, "update");
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("修改用户状态（封禁、解锁）")
    @XApiPost
    public XServiceResult updateSysUserStatus(XContext context, UpdateSysUserInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysUserMapper mapper = context.getBean(SysUserMapper.class);
            QueryWrapper<SysUserEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysUserEnt::getId, input.getId());
            SysUserEnt entity = mapper.selectOne(queryWrapper);
            if (entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            entity.setStatus(input.getStatus());
            entity.setModifyTime(new Date());
            PpsUserSession session = context.getSession(PpsUserSession.class);
            entity.setModifyById(session.getId());
            entity.setModifyByName(session.getUserName());
            mapper.updateById(entity);
            if (input.getStatus() == 0) {
                //删除登录日志错误信息
                SysUserLoginLogMapper userLoginLogMapper = context.getBean(SysUserLoginLogMapper.class);
                QueryWrapper<SysUserLoginLogEnt> queryLogWrapper = new QueryWrapper<>();
                queryLogWrapper.lambda().eq(SysUserLoginLogEnt::getLoginName, entity.getLoginName());
                userLoginLogMapper.delete(queryLogWrapper);
            }
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    public XServiceResult updateSysUserOrganizationRel(XContext context, String userId, String ouId, PpsUserSession session, String flag) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            //修改用户和组织机构关系为失效
            SysUserOrganizationRelMapper relMapper = context.getBean(SysUserOrganizationRelMapper.class);
            Date date = new Date();
            QueryWrapper<SysUserOrganizationRelEnt> userOuQueryWrapper = new QueryWrapper<>();
            userOuQueryWrapper.lambda().eq(SysUserOrganizationRelEnt::getUserId, userId);
            userOuQueryWrapper.lambda().le(true, SysUserOrganizationRelEnt::getEffectTime, date);
            userOuQueryWrapper.lambda().ge(true, SysUserOrganizationRelEnt::getEndTime, date);
            SysUserOrganizationRelEnt relEntity = relMapper.selectOne(userOuQueryWrapper);
            if (relEntity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            //如果是修改就新增一条，删除的话不需要新增
            if ("update".equals(flag)) {
                //如果不相等改为失效
                if (!relEntity.getOuId().equals(ouId)) {
                    relEntity.setEndTime(date);
                    if (session != null) {
                        relEntity.setModifyById(session.getId());
                        relEntity.setModifyByName(session.getUserName());
                    }
                    relEntity.setModifyTime(date);
                    relMapper.updateById(relEntity);
                    //新增一条记录
                    insertSysUserOrganizationRel(context, userId, ouId, session);
                }
            } else {
                relEntity.setEndTime(date);
                if (session != null) {
                    relEntity.setModifyById(session.getId());
                    relEntity.setModifyByName(session.getUserName());
                }
                relEntity.setModifyTime(date);
                relMapper.updateById(relEntity);
            }
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("删除用户")
    @XApiPost
    public XServiceResult deleteSysUser(XContext context, DeleteSysUserInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysUserMapper mapper = context.getBean(SysUserMapper.class);
            QueryWrapper<SysUserEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysUserEnt::getId, input.getId());
            SysUserEnt entity = mapper.selectOne(queryWrapper);
            if (entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            mapper.deleteById(entity);
            //修改用户和组织机构表为失效
            updateSysUserOrganizationRel(context, input.getId(), null, context.getSession(PpsUserSession.class), "delete");
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetSysUserOutput> getSysUser(XContext context, GetSysUserInput input) {
        SysUserMapper mapper = context.getBean(SysUserMapper.class);
        QueryWrapper<SysUserEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserEnt::getId, input.getId());
        SysUserEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysUserOutput output = new GetSysUserOutput();
        XCopyUtils.copyObject(entity, output);
        output.setPhone(UserInfoUtil.desensitization(entity.getPhone()));
        output.setIdentifyNo(UserInfoUtil.desensitization(entity.getIdentifyNo()));
        output.setEmail(UserInfoUtil.desensitization(entity.getEmail()));
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySysUserOutput> querySysUser(XContext context, QuerySysUserInput input) {
        SysUserMapper mapper = context.getBean(SysUserMapper.class);
        QueryWrapper<SysUserEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(SysUserEnt::getId, input.getId())
                .like(SysUserEnt::getUserName, input.getUserName())
                .like(SysUserEnt::getUserDomain, input.getUserDomain())
                .like(SysUserEnt::getUserCode, input.getUserCode())
                .eq(SysUserEnt::getUserType, input.getUserType())
                .like(SysUserEnt::getPhone, input.getPhone())
                .like(SysUserEnt::getEmail, input.getEmail())
                .like(SysUserEnt::getLoginName, input.getLoginName())
                .eq(SysUserEnt::getIsSuperAdmin, input.getIsSuperAdmin())
                .like(SysUserEnt::getIdentifyNo, input.getIdentifyNo())
                .like(SysUserEnt::getRemark, input.getRemark())
                .eq(SysUserEnt::getEndDate, input.getEndDate())
                .like(SysUserEnt::getPassword, input.getPassword())
                .like(SysUserEnt::getDuty, input.getDuty())
                .eq(SysUserEnt::getSignImage, input.getSignImage())
                .like(SysUserEnt::getPromptMode, input.getPromptMode())
                .eq(SysUserEnt::getCert, input.getCert())
                .eq(SysUserEnt::getSort, input.getSort())
                .eq(SysUserEnt::getCaBeginDate, input.getCaBeginDate())
                .eq(SysUserEnt::getCaEndDate, input.getCaEndDate())
                .like(SysUserEnt::getIamLoginName, input.getIamLoginName())
                .eq(SysUserEnt::getLastLoginTime, input.getLastLoginTime())
                .like(SysUserEnt::getJobDesc, input.getJobDesc())
                .like(SysUserEnt::getFaxNo, input.getFaxNo())
                .like(SysUserEnt::getPostalCode, input.getPostalCode())
                .like(SysUserEnt::getAddress, input.getAddress())
                .like(SysUserEnt::getCreateById, input.getCreateById())
                .like(SysUserEnt::getCreateByName, input.getCreateByName())
                .eq(SysUserEnt::getCreateTime, input.getCreateTime())
                .like(SysUserEnt::getModifyById, input.getModifyById())
                .like(SysUserEnt::getModifyByName, input.getModifyByName())
                .eq(SysUserEnt::getModifyTime, input.getModifyTime());
        return XMapperHelper.query(mapper, input, queryWrapper, QuerySysUserOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySysUserOutput> dynamicQuerySysUser(XContext context, CustomQueryInput input) {
        SysUserMapper mapper = context.getBean(SysUserMapper.class);
        return XMapperHelper.query(mapper, input, SysUserEnt.class, QuerySysUserOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetSysUserViewOutput> getSysUserView(XContext context, GetSysUserViewInput input) {
        SysUserViewMapper mapper = context.getBean(SysUserViewMapper.class);
        SysUserView record = new SysUserView();
        XCopyUtils.copyObject(input, record);
        SysUserView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysUserViewOutput output = new GetSysUserViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySysUserViewOutput> querySysUserView(XContext context, QuerySysUserViewInput input) {
        SysUserViewMapper mapper = context.getBean(SysUserViewMapper.class);
        SysUserView record = new SysUserView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysUserView> list = mapper.selectList(record);
        PageInfo<SysUserView> pageInfo = new PageInfo<>(list);
        List<QuerySysUserViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysUserViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }


    @XText("根据组织机构id分页获取用户列表")
    @XApiPost
    public XPageResult<QuerySysUserViewOutput> getUserListByOuIdPage(XContext context, GetUserListByOuIdInput input) {
        SysUserViewMapper mapper = context.getBean(SysUserViewMapper.class);
        SysUserView record = new SysUserView();
        XCopyUtils.copyObject(input, record);
        record.setEndTime(new Date());
        if (Strings.isBlank(record.getOuId())) {
            SysOrganizationViewMapper organizationViewMapper = context.getBean(SysOrganizationViewMapper.class);
            SysOrganizationView sysOrganizationView = new SysOrganizationView();
            sysOrganizationView.setEndTime(new Date());
            sysOrganizationView.setIsDeleted(SysOrganizationCode.NORMAL);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            List<SysOrganizationView> viewList;
            if (input.getShowAll() == null || !input.getShowAll()) {
                sysOrganizationView.setId(session.getOuId());
                viewList = organizationViewMapper.selectOuByOuId(sysOrganizationView);
                if (viewList.isEmpty()) return XPageResult.success(new ArrayList<>(), input, 0);
                List<String> ouIds = viewList.stream().map(SysOrganizationView::getId).collect(Collectors.toList());
                record.setOuIds(ouIds);
            }
        }
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysUserView> list = mapper.selectSysUserListByOuId(record);
        PageInfo<SysUserView> pageInfo = new PageInfo<>(list);
        List<QuerySysUserViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysUserViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("交易平台获取用户")
    @XApiPost
    public XListResult<QuerySysUserByTradeOutput> querySysUserByTrade(XContext context, QuerySysUserByTradeInput input) {
        String userParam = input.getUserParam();
        if (StringUtils.isBlank(userParam)) {
            return XListResult.error(context, LoginError.Trade_ParamIsNull);
        }
        String userInfo;
        try {
            String privateKey = LoginConfig.rsaPrivateKey.findValue(context);
            privateKey = XEncryptUtils.decrypt(privateKey);
            userInfo = RSAUtil.RsaDecrypt(userParam, privateKey);
        } catch (Exception e) {
            context.getLogger().error("参数解密失败", e);
            return XListResult.error(context, LoginError.Trade_ParamRSAError);
        }
        TradeUserInfo tradeUserInfo;
        try {
            tradeUserInfo = XJsonUtils.toObject(userInfo, TradeUserInfo.class);
            if (!(tradeUserInfo != null && StringUtils.isNotBlank(tradeUserInfo.getUserName()))) {
                return XListResult.error(context, LoginError.Trade_ParamFormatError);
            }
        } catch (Exception e) {
            context.getLogger().error("参数格式有误", e);
            return XListResult.error(context, LoginError.Trade_ParamFormatError);
        }
        SysUserViewMapper mapper = context.getBean(SysUserViewMapper.class);
        SysUserView userView = new SysUserView();
        userView.setUserName(tradeUserInfo.getUserName());
        List<SysUserTradeView> userEntList = mapper.selectUserByTrade(userView);
        List<QuerySysUserByTradeOutput> outputList = XCopyUtils.copyNewList(userEntList, QuerySysUserByTradeOutput.class);
        return XListResult.success(outputList);
    }

    @XText("交易平台用户验证接口")
    @XApiPost
    public XSingleResult<CheckSysUserByTradeOutput> checkSysUserByTrade(XContext context, CheckSysUserByTradeInput input) {
        String userParam = input.getUserParam();
        if (StringUtils.isBlank(userParam)) {
            return XSingleResult.error(context, LoginError.Trade_ParamIsNull);
        }
        String userInfo;
        try {
            String privateKey = LoginConfig.rsaPrivateKey.findValue(context);
            privateKey = XEncryptUtils.decrypt(privateKey);
            userInfo = RSAUtil.RsaDecrypt(userParam, privateKey);
        } catch (Exception e) {
            context.getLogger().error("参数解密失败", e);
            return XSingleResult.error(context, LoginError.Trade_ParamRSAError);
        }
        TradeUserInfo tradeUserInfo;
        try {
            tradeUserInfo = XJsonUtils.toObject(userInfo, TradeUserInfo.class);
            if (!(tradeUserInfo != null && StringUtils.isNotBlank(tradeUserInfo.getUserName()))) {
                return XSingleResult.error(context, LoginError.Trade_ParamFormatError);
            }
        } catch (Exception e) {
            context.getLogger().error("参数格式有误", e);
            return XSingleResult.error(context, LoginError.Trade_ParamFormatError);
        }
        SysUserMapper mapper = context.getBean(SysUserMapper.class);
        QueryWrapper<SysUserEnt> wrapper = new QueryWrapper();
        wrapper.lambda().eq(SysUserEnt::getLoginName, tradeUserInfo.getUserName())
                .eq(SysUserEnt::getIsTrade, 1);
        SysUserEnt sysUserEnt = mapper.selectOne(wrapper);
        CheckSysUserByTradeOutput out = new CheckSysUserByTradeOutput();
        if (sysUserEnt == null) {
            out.setExist(false);
            out.setCanBeBind(false);
        } else {
            out.setExist(true);
            if (sysUserEnt.getIsTrade() != null && sysUserEnt.getIsTrade() == 1) {
                out.setCanBeBind(true);
            } else {
                out.setCanBeBind(false);
            }
        }
        return XSingleResult.success(out);
    }


    @XText("根据用户ouIdList获取用户列表发送短信")
    @XApiAnonymous
    @XApiPost
    public XListResult<GetSysUserOutput> querySysUserListByOuIds(XContext context, GetSysUserListByOuInput input) {
        SysUserViewMapper mapper = context.getBean(SysUserViewMapper.class);
        List<SysUserView> list;
        if (input.getOuIds().size() > 0) {
            list = mapper.getSysUserListByOuIds(input);
            list.forEach(user -> {
                String phone = user.getPhone();
                if (StringUtils.isNotBlank(phone))
                    user.setPhone(UserInfoUtil.decrypt(phone));
            });
            List<GetSysUserOutput> outputList = XCopyUtils.copyNewList(list, GetSysUserOutput.class);
            return XListResult.success(outputList);
        }
        return null;
    }

    @XText("根据用户ouId和userName获取用户列表发送短信")
    @XApiAnonymous
    @XApiPost
    public XPageResult<QuerySysUserListByOuIdorNameOutput> getSysUserListByOuIdOrUserName(XContext context, QuerySysUserListByOuIdorNameInput input) {
        SysUserViewMapper mapper = context.getBean(SysUserViewMapper.class);
        GetSysUserListByOuInput record = new GetSysUserListByOuInput();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysUserView> list;
        list = mapper.getSysUserListByOuIdOrUserName(record);
        list.forEach(user -> {
            String phone = user.getPhone();
            if (StringUtils.isNotBlank(phone))
                user.setPhone(UserInfoUtil.decrypt(phone));
        });
        PageInfo<SysUserView> pageInfo = new PageInfo<>(list);
        List<QuerySysUserListByOuIdorNameOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysUserListByOuIdorNameOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 根据当前登录人,获取用户名  员工编号  邮箱  单位
     *
     * @param context
     * @return
     */
    @XApiAnonymous
    @XApiGet
    public XSingleResult<CurrentSysUserOutput> getSysUserByLoginUser(XContext context) {
        XUser user = context.getUser();
        String code = user.getCode();
        SysUserViewMapper mapper = context.getBean(SysUserViewMapper.class);
        CurrentSysUserView sysUserView = mapper.getSysUserAndOuByUserId(code);
        CurrentSysUserOutput output = new CurrentSysUserOutput();
        XCopyUtils.copyObject(sysUserView, output);
//        output.setPhone(UserInfoUtil.desensitization(entity.getPhone()));
//        output.setIdentifyNo(UserInfoUtil.desensitization(entity.getIdentifyNo()));
        output.setEmail(UserInfoUtil.desensitization(output.getEmail()));
        return XSingleResult.success(output);
    }
}
