package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.QuerySysDictionaryViewInput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import pps.core.base.entity.BaseStationOuRelEnt;
import pps.core.base.entity.BaseStationOuRelView;
import pps.core.base.mapper.BaseStationOuRelMapper;
import pps.core.base.mapper.BaseStationOuRelViewMapper;
import pps.core.base.service.data.base_station_ou_rel.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;

@XText("场站组织机构关系接口")
@XService
public class BaseStationOuRelService {

    @XText("创建场站组织机构关系")
    @XApiPost
    public XServiceResult createBaseStationOuRel(XContext context, CreateBaseStationOuRelInput input) {
        BaseStationOuRelMapper mapper = context.getBean(BaseStationOuRelMapper.class);
        BaseStationOuRelEnt entity = new BaseStationOuRelEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("修改场站组织机构关系")
    @XApiPost
    public XServiceResult updateBaseStationOuRel(XContext context, UpdateBaseStationOuRelInput input) {
        BaseStationOuRelMapper mapper = context.getBean(BaseStationOuRelMapper.class);
        QueryWrapper<BaseStationOuRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseStationOuRelEnt::getId, input.getId());
        BaseStationOuRelEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除场站组织机构关系")
    @XApiPost
    public XServiceResult deleteBaseStationOuRel(XContext context, DeleteBaseStationOuRelInput input) {
        BaseStationOuRelMapper mapper = context.getBean(BaseStationOuRelMapper.class);
        QueryWrapper<BaseStationOuRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseStationOuRelEnt::getId, input.getId());
        BaseStationOuRelEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XText("根据id铲鲟场站组织机构关系")
    @XApiPost
    public XSingleResult<GetBaseStationOuRelOutput> getBaseStationOuRel(XContext context, GetBaseStationOuRelInput input) {
        BaseStationOuRelMapper mapper = context.getBean(BaseStationOuRelMapper.class);
        QueryWrapper<BaseStationOuRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseStationOuRelEnt::getId, input.getId());
        BaseStationOuRelEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseStationOuRelOutput output = new GetBaseStationOuRelOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("分页条件查询场站组织机构关系")
    @XApiPost
    public XPageResult<QueryBaseStationOuRelOutput> queryBaseStationOuRel(XContext context, QueryBaseStationOuRelInput input) {
        BaseStationOuRelMapper mapper = context.getBean(BaseStationOuRelMapper.class);
        QueryWrapper<BaseStationOuRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(BaseStationOuRelEnt::getId, input.getId())
                .like(BaseStationOuRelEnt::getStationId, input.getStationId())
                .like(BaseStationOuRelEnt::getOuId, input.getOuid())
                .eq(BaseStationOuRelEnt::getEffectTime, input.getEffectTime())
                .eq(BaseStationOuRelEnt::getEndTime, input.getEndTime())
                .eq(BaseStationOuRelEnt::getSort, input.getSort())
                .like(BaseStationOuRelEnt::getCreateById, input.getCreateById())
                .like(BaseStationOuRelEnt::getCreateByName, input.getCreateByName())
                .eq(BaseStationOuRelEnt::getCreateTime, input.getCreateTime())
                .like(BaseStationOuRelEnt::getModifyById, input.getModifyById())
                .like(BaseStationOuRelEnt::getModifyByName, input.getModifyByName())
                .eq(BaseStationOuRelEnt::getModifyTime, input.getModifyTime());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseStationOuRelOutput.class);
    }

    @XApiPost
    public XPageResult<QueryBaseStationOuRelOutput> dynamicQueryBaseStationOuRel(XContext context, CustomQueryInput input) {
        BaseStationOuRelMapper mapper = context.getBean(BaseStationOuRelMapper.class);
        return XMapperHelper.query(mapper, input, BaseStationOuRelEnt.class, QueryBaseStationOuRelOutput.class);
    }

    @XApiPost
    public XSingleResult<GetBaseStationOuRelViewOutput> getBaseStationOuRelView(XContext context, GetBaseStationOuRelViewInput input) {
        BaseStationOuRelViewMapper mapper = context.getBean(BaseStationOuRelViewMapper.class);
        BaseStationOuRelView record = new BaseStationOuRelView();
        XCopyUtils.copyObject(input, record);
        BaseStationOuRelView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseStationOuRelViewOutput output = new GetBaseStationOuRelViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiPost
    public XPageResult<QueryBaseStationOuRelViewOutput> queryBaseStationOuRelView(XContext context, QueryBaseStationOuRelViewInput input) {
        BaseStationOuRelViewMapper mapper = context.getBean(BaseStationOuRelViewMapper.class);
        BaseStationOuRelView record = new BaseStationOuRelView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BaseStationOuRelView> list = mapper.selectList(record);
        PageInfo<BaseStationOuRelView> pageInfo = new PageInfo<>(list);
        List<QueryBaseStationOuRelViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseStationOuRelViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XApiGet
    @XText("获取组织与场站关系")
    public XPageResult<GetBaseStationOuRelOutput> getBaseStationOuRelList(XContext context, QueryBaseStationOuRelInput input) {
        BaseStationOuRelViewMapper mapper = context.getBean(BaseStationOuRelViewMapper.class);
        BaseStationOuRelView record = new BaseStationOuRelView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BaseStationOuRelView> list = mapper.selectList(record);
        List<QuerySysDictionaryViewOutput> dictionaryViewOutputs = getMenuDict(context, "MEDIUM_TYPE");
        list.stream().forEach(item -> {
            if (XStringUtils.isNotEmpty(item.getMediumType())) {
                String dictName = getDictName(item.getMediumType(), dictionaryViewOutputs);
                if (XStringUtils.isEmpty(dictName))
                    item.setMediumTypeName(item.getStationName());
                else
                    item.setMediumTypeName(item.getStationName() + "-" + dictName);
            } else
                item.setMediumTypeName(item.getStationName());
        });
        PageInfo<BaseStationOuRelView> pageInfo = new PageInfo<>(list);
        List<GetBaseStationOuRelOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), GetBaseStationOuRelOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    public List<QuerySysDictionaryViewOutput> getMenuDict(XContext context, String alias) {
        SystemDictionaryService dictionaryService = context.getBean(SystemDictionaryService.class);
        QuerySysDictionaryViewInput dictInput = new QuerySysDictionaryViewInput();
        dictInput.setAlias(alias);
        dictInput.setPage(1);
        dictInput.setLimit(999);
        XListResult<QuerySysDictionaryViewOutput> dictPageResult = dictionaryService.queryChildSysDictionarysByParentAlias(context, dictInput);
        dictPageResult.throwIfFail();
        return dictPageResult.getResult();
    }

    public String getDictName(String dictKey, List<QuerySysDictionaryViewOutput> dictList) {
        if (StringUtils.isBlank(dictKey) || CollectionUtils.isEmpty(dictList)) {
            return " ";
        }
        for (QuerySysDictionaryViewOutput dict : dictList) {
            if (dict.getDicKey().equals(dictKey)) {
                return dict.getDicName();
            }
        }
        return "";
    }
}