package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.IBaseWellheadCloudService;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadInput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadOutput;
import pps.core.base.entity.BaseWellheadEnt;
import pps.core.base.mapper.BaseWellheadMapper;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;

import java.util.List;

/**
 * 井口配置Cloud模块
 *
 * @author ZWT
 * @date 2023/09/13 17:34
 */
@XService
public class BaseWellheadCloudServiceImpl implements IBaseWellheadCloudService {

    /**
     * 井口配置Cloud模块--条件查询井口基本信息列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBaseWellheadOutput}>
     */
    @Override
    public XListResult<DynamicQueryBaseWellheadOutput> queryBaseWellheadListByParam(XContext context, DynamicQueryBaseWellheadInput input) {
        String wellheadId = input.getWellheadId();
        List<String> wellheadIds = input.getWellheadIds();
        List<String> wellNumbers = input.getWellNumbers();
        String ouId = input.getOuId();
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        List<BaseWellheadEnt> list = mapper.selectList(
                new LambdaQueryWrapper<BaseWellheadEnt>()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(CharSequenceUtil.isNotBlank(wellheadId), BaseModel::getId, wellheadId)
                        .eq(CharSequenceUtil.isNotBlank(ouId), BaseWellheadEnt::getOuId, ouId)
                        .in(CollUtil.isNotEmpty(wellheadIds), BaseModel::getId, wellheadIds)
                        .in(CollUtil.isNotEmpty(wellNumbers), BaseWellheadEnt::getWellNumber, wellNumbers)
        );
        return XListResult.success(XCopyUtils.copyNewList(
                list,
                DynamicQueryBaseWellheadOutput.class
        ));
    }

    /**
     * 井口配置Cloud模块--查功率最小的井口
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link DynamicQueryBaseWellheadOutput}>
     */
    @Override
    public XSingleResult<DynamicQueryBaseWellheadOutput> getMinPowerWellByIds(XContext context, DynamicQueryBaseWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        BaseWellheadEnt one = mapper.selectOne(new LambdaQueryWrapper<BaseWellheadEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .in(BaseModel::getId, input.getWellheadIds())
                .orderByDesc(BaseWellheadEnt::getServiceRating)
                .last("LIMIT 1")
        );
        return XSingleResult.success(XCopyUtils.copyNewObject(one, DynamicQueryBaseWellheadOutput.class));
    }
}
