package pps.core.base.service;

import com.google.common.base.Splitter;
import com.google.common.base.Strings;
import pps.core.base.service.data.MyCounterOutput;
import pps.core.common.job.CounterJob5S;
import pps.core.common.utils.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.XStartup;
import xstartup.base.util.XJsonUtils;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiGet;

import java.util.List;
import java.util.Set;

@XService
public class CounterService {

    public static final String LINE_SEPERATOR = System.getProperty("line.separator");

    /**
     * 读取全局计数器，用于问题排查与诊断
     *
     * @param context
     * @return
     */
//    @XApiAnonymous
    @XText("获取全局计数器")
    @XApiGet
    public XSingleResult<MyCounterOutput> getGlobalCounter(XContext context) {
        CounterBuilder.globalCounterBuilder.setFieldValue("__TASKS_RUNNING__", GlobalExecutorService.INSTANCE.getRunningTasksCount() + "");
        CounterBuilder.globalCounterBuilder.setFieldValue("__TASKS_DONE__", GlobalExecutorService.INSTANCE.getTasksCount() + "");
        CounterBuilder.globalCounterBuilder.setFieldValue("__TASKS_RUNNING(SCADA)___", ScadaExecutorService.INSTANCE.getRunningTasksCount() + "");
        CounterBuilder.globalCounterBuilder.setFieldValue("__TASKS_DONE(SCADA)__", ScadaExecutorService.INSTANCE.getTasksCount() + "");
        CounterBuilder.globalCounterBuilder.setFieldValue("__TRACEBUILDERS__", TraceHelper.getTraceBuildersCount() + "");
        MyCounterOutput output = new MyCounterOutput();
        output.put("__Time__", DateUtil.getCurrentDateTime());
        output.merge(CounterBuilder.globalCounterBuilder.toDynObj());
        output.put("__IPS__", IpUtils.getHostIPs(true));
        output.put("__THREADS__", ThreadHelper.buildThreadsSimpleInfos(null));
        output.put("__ComponentInfo__", XJsonUtils.toJson(ManifestComponentInfoHelper.getManifestComponentInfo("pps", "pps-core-base")).replace("\"", ""));
        return XSingleResult.success(output);
    }

    /**
     * 读取集群计数器
     *
     * @param context
     * @return
     */
    @XText("获取集群全局计数器")
    @XApiGet
    public XSingleResult<MyCounterOutput> getClusterGlobalCounter(XContext context) {
        try {
            MyCounterOutput finalOutput = new MyCounterOutput();
            String cacheType = XStartup.getCurrent().getProperty("x.cache.type");
            String appClusterName = CounterJob5S.getAppClusterName();
            String appClusterLockName = CounterJob5S.getAppClusterLockName();
            if ("redis-cluster".equals(cacheType)) {
                JedisHelper.consumeDefalutJedisCluster(jedis -> {
                    Set<String> nodeKeys = jedis.smembers(appClusterName);
                    finalOutput.put("NODES", XJsonUtils.toJson(nodeKeys));
                    finalOutput.put("MASTER", jedis.get(appClusterLockName));
                    nodeKeys.forEach(k -> {
                        if (jedis.exists(k)) {
                            String dynObjJson = jedis.get(k);
                            DynObj dynObj = XJsonUtils.toObject(dynObjJson, DynObj.class);
                            String owner = dynObj.getString(CounterBuilder.COUNTER_NAME_OWNER);
                            if (Strings.isNullOrEmpty(owner)) {
                                owner = dynObj.getString(CounterBuilder.COUNTER_NAME_INSTANCE);
                            }
                            finalOutput.put(owner, dynObj);
                        }
                    });
                });
            } else {
                JedisHelper.consumeDefalutJedis(jedis -> {
                    Set<String> nodeKeys = jedis.smembers(appClusterName);
                    finalOutput.put("NODES", XJsonUtils.toJson(nodeKeys));
                    finalOutput.put("MASTER", jedis.get(appClusterLockName));
                    nodeKeys.forEach(k -> {
                        if (jedis.exists(k)) {
                            String dynObjJson = jedis.get(k);
                            DynObj dynObj = XJsonUtils.toObject(dynObjJson, DynObj.class);
                            String owner = dynObj.getString(CounterBuilder.COUNTER_NAME_OWNER);
                            if (Strings.isNullOrEmpty(owner)) {
                                owner = dynObj.getString(CounterBuilder.COUNTER_NAME_INSTANCE);
                            }
                            finalOutput.put(owner, dynObj);
                        }
                    });
                });
            }
            return XSingleResult.success(finalOutput);
        } catch (Exception ex) {
            CounterBuilder.globalCounterBuilder.addCounter(ex, this.getClass());
            return XSingleResult.error(200, ex.getMessage());
        }
    }

    @XText("threads")
    @XApiGet
    public XSingleResult<MyCounterOutput> getThreads(XContext context) {
        StringBuilder sb = new StringBuilder();
        ThreadHelper.buildThreadsInfos(sb);
        List<String> list = Splitter.on(LINE_SEPERATOR).splitToList(sb.toString());
        MyCounterOutput output = new MyCounterOutput();
        output.put("threads", list);
        return XSingleResult.success(output);
    }
}