package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.io.FileUtil;
import com.alibaba.excel.EasyExcelFactory;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import pps.core.base.config.excel.CascadeWriteHandler;
import pps.core.base.config.excel.ExcelStyleTool;
import pps.core.base.service.data.base_excel.ExcelPhotovoltaicPlantTemplate;
import pps.core.base.utils.EasyExcelUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.tool.XStorageTool;
import xstartup.data.XFileResult;
import xstartup.data.XFileType;
import xstartup.feature.api.annotation.XApiGet;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * excel导入导出
 *
 * @author ZWT
 * @date 2024/05/07 09:49
 */
@XText("Excel")
@XService
public class ExcelService {

    /**
     * 光伏电站模板导出
     *
     * @param context 上下文
     * @return {@link XFileResult}
     */
    @XText("光伏电站模板导出")
    @XApiGet(anonymous = true)
    public XFileResult templatePhotovoltaicPlant(XContext context) {
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/光伏电站模板.xlsx");
        File file = FileUtil.touch(fileSavePath);
        // 准备要写出的数据（这里可以从数据库中查询出来后再进行）
        // 查询所有的省名称
        List<String> provNameList = new ArrayList<>();
        provNameList.add("浙江省");
        provNameList.add("广东省");
        // 整理数据，放入一个Map中，mapkey存放父地点，value 存放该地点下的子区域
        Map<String, List<String>> siteMap = new HashMap<>();
        siteMap.put("浙江省", CollUtil.newArrayList("杭州市", "金华市", "宁波市"));
        siteMap.put("广东省", CollUtil.newArrayList("广州市", "深圳市", "韶光市"));
        ExcelWriter excelWriter = EasyExcelFactory.write(file)
                .registerWriteHandler(ExcelStyleTool.getStyleStrategy())
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                .registerWriteHandler(new CascadeWriteHandler(provNameList, siteMap))
                .autoCloseStream(Boolean.TRUE)
                .build();
        WriteSheet writeSheet = EasyExcelUtil.writeSelectedSheet(ExcelPhotovoltaicPlantTemplate.class, 0, "光伏电站");
        excelWriter.write(new ArrayList<String>(), writeSheet);
        excelWriter.finish();
        return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
    }
}
