package pps.core.base.service.data.third_weather_data;

import com.alibaba.fastjson.annotation.JSONField;
import lombok.Data;
import xstartup.annotation.XText;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 第三方气象数据
 *
 * @author ZWT
 * @date 2024/04/23
 */
@Data
public class ThirdApiWeatherData {

    @XText("太阳直接辐射[W/m2]")
    private BigDecimal dni;

    @XText("太阳总辐射（水平面）[W/m2]")
    private BigDecimal ghi;

    @XText("太阳散射辐射（水平面）[W/m2]")
    private BigDecimal dhi;

    @XText("预报时间")
    @JSONField(name = "time")
    private Date forecastTime;

    @XText("气温 [C]")
    private BigDecimal tem;

    @XText("相对湿度 [%]")
    private BigDecimal rhu;

    @XText("风速（地面10米）[m/s]")
    private BigDecimal wns;

    @XText("风向角度（地面10米）[degree]")
    private Integer wnd;

    @XText("风力等级（地面10米）")
    @JSONField(name = "wns_grd")
    private Integer wnsGrd;

    @XText("阵风风速（地面10米）[m/s]")
    private BigDecimal gust;

    @XText("降水量（1小时累计）[mm]")
    private BigDecimal pre;

    @XText("地面气压 [hPa]")
    @JSONField(name = "prs_qfe")
    private BigDecimal prsQfe;

    @XText("能见度 [km]")
    private BigDecimal vis;

    @XText("云量 [%]")
    private BigDecimal clo;

    @XText("紫外线UV辐照度（1小时累计）[kWh/m2]")
    private BigDecimal uvb;

    @XText("紫外线UV辐照指数")
    private Integer uvi;

    @XText("降水相态：(0无降水, 1雨, 3冻雨, 5雪, 6湿雪, 7雨夹雪, 8冰雹)")
    private Integer phs;

    @XText("天气现象代码")
    private Integer wep;

    @XText("风向角度（地面100米）[degree]")
    @JSONField(name = "wnd_100m")
    private Integer wnd100m;

    @XText("风速（地面100米）[m/s]")
    @JSONField(name = "wns_100m")
    private BigDecimal wns100m;

    @XText("风力等级（地面100米）")
    @JSONField(name = "wns_grd_100m")
    private Integer wnsGrd100m;

    @XText("天气数据更新时间")
    private Date updateTime;

    @XText("数据创建时间")
    private Date createTime;

    @XText("经度(°)")
    private BigDecimal longitude;

    @XText("纬度(°)")
    private BigDecimal latitude;
}
