package pps.core.task.job;

import cn.hutool.core.util.ObjectUtil;
import pps.cloud.space.service.ISpaceOptimizeLongCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

/**
 * 中期间开优化定时任务
 *
 * @author ZWT
 * @date 2023/09/13 13:59
 */
@XText("中期间开优化定时任务")
@XService
public class SpaceOptimizeLongJob implements XJob {

    /**
     * 每天00:15执行
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    @XCronTrigger(value = TaskConstant.EVERY_DAY_15)
    @Override
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ SpaceOptimizeLongJob start:{}", System.currentTimeMillis());
        String key = TaskConstant.TASK_LOCK_KEY + "SpaceOptimizeLongJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            TaskLockCache cache = new TaskLockCache();
            cache.setRedisKey(key);
            cache.setRedisValue(key);
            TaskLockCache.set(xContext, cache);
            try {
                ISpaceOptimizeLongCloudService service = xContext.getBean(ISpaceOptimizeLongCloudService.class);
                XServiceResult result = service.optimizeLongJob(xContext);
                result.throwIfFail();
            } catch (Exception e) {
                xContext.getLogger().error("------ SpaceOptimizeLongJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ SpaceOptimizeLongJob end:{}", System.currentTimeMillis());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}