package pps.core.task.job;

import cn.hutool.core.util.ObjectUtil;
import pps.cloud.prediction.service.IThirdDataAccessCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

/**
 * 第三方单井平均有功功率定时任务
 *
 * @author ZWT
 * @date 2024/04/01 17:01
 */
@XText("第三方单井平均有功功率定时任务")
@XService
public class ThirdWellAvgActivePowerJob implements XJob {

    /**
     * 十分钟一次隔3分钟
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    @XCronTrigger(value = TaskConstant.TEN_MINUTES_3)
    @Override
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ ThirdWellAvgActivePowerJob start:{}", System.currentTimeMillis());
        String key = TaskConstant.TASK_LOCK_KEY + "ThirdWellAvgActivePowerJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            TaskLockCache cache = new TaskLockCache();
            cache.setRedisKey(key);
            cache.setRedisValue(key);
            TaskLockCache.set(xContext, cache);
            try {
                IThirdDataAccessCloudService service = xContext.getBean(IThirdDataAccessCloudService.class);
                XServiceResult result = service.thirdWellAvgActivePowerJob(xContext);
                result.throwIfFail();
            } catch (Exception e) {
                xContext.getLogger().error("------ ThirdWellAvgActivePowerJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ ThirdWellAvgActivePowerJob end:{}", System.currentTimeMillis());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}
