package pps.core.task.job;

import cn.hutool.core.util.ObjectUtil;
import pps.cloud.base.service.IBaseWeatherCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

/**
 * 天气API数据邮件接收定时任务
 *
 * @author ZWT
 * @date 2024/04/22 17:24
 */
@XText("天气API数据邮件接收定时任务")
@XService
public class WeatherApiReceiveJob implements XJob {

    /**
     * 早/晚九点十五分执行
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    @XCronTrigger(value = TaskConstant.NINE_OF_DAY)
    @Override
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ WeatherApiReceiveJob start:{}", System.currentTimeMillis());
        String key = TaskConstant.TASK_LOCK_KEY + "WeatherApiReceiveJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            TaskLockCache cache = new TaskLockCache();
            cache.setRedisKey(key);
            cache.setRedisValue(key);
            TaskLockCache.set(xContext, cache);
            try {
                IBaseWeatherCloudService service = xContext.getBean(IBaseWeatherCloudService.class);
                XServiceResult result = service.weatherApiDataProcess(xContext);
                result.throwIfFail();
            } catch (Exception e) {
                xContext.getLogger().error("------ WeatherApiReceiveJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ WeatherApiReceiveJob end:{}", System.currentTimeMillis());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}
