package pps.core.prediction.service;

import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import pps.core.common.cache.ThirdPartyConfigCache;
import pps.core.prediction.constant.ThirdPartyApiConstant;
import pps.core.prediction.entity.ThirdActivePowerView;
import pps.core.prediction.enums.BusinessError;
import pps.core.prediction.mapper.ThirdActivePowerViewMapper;
import pps.core.prediction.service.data.third_active_power.GetThirdActivePowerOutput;
import pps.core.prediction.service.data.third_active_power.GetThirdSanDieLoginInfoOutput;
import pps.core.prediction.service.data.third_active_power_daily_update.GetThirdActivePowerDailyUpdateInput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XHttpUtils;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiPost;

import java.util.List;

/**
 * 第三方有功功率
 *
 * @author ZWT
 * @date 2024/03/19
 */
@XText("第三方有功功率")
@XService
public class ThirdActivePowerService {

    /**
     * 有功功率--用能曲线
     * POST /prediction/third-active-power/get-active-power-list
     * 接口ID：157089363
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-157089363
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link GetThirdActivePowerOutput}>
     */
    @XText("有功功率--用能曲线")
    @XApiPost
    public XListResult<GetThirdActivePowerOutput> getActivePowerList(XContext context, GetThirdActivePowerDailyUpdateInput input) {
        ThirdActivePowerViewMapper mapper = context.getBean(ThirdActivePowerViewMapper.class);
        List<ThirdActivePowerView> list = mapper.selectEnergyUseCurve(ThirdActivePowerView.builder()
                .stationName(input.getStationName())
                .createDate(input.getQueryDate())
                .build());
        return XListResult.success(XCopyUtils.copyNewList(list, GetThirdActivePowerOutput.class));
    }

    /**
     * 获取三叠登录信息
     *
     * @param context 上下文
     * @return {@link XSingleResult}<{@link GetThirdSanDieLoginInfoOutput}>
     */
    @XText("第三方--获取三叠登录信息")
    @XApiPost
    public XSingleResult<GetThirdSanDieLoginInfoOutput> getThirdSanDieLoginInfo(XContext context) {
        String loginInfo;
        ThirdPartyConfigCache exist = ThirdPartyConfigCache.exist(context, ThirdPartyApiConstant.CQ_TOKEN_SAN_DIE_CACHE_KEY);
        if (ObjectUtil.isNull(exist)) {
            loginInfo = this.getSanDieToken(context);
        } else {
            long duration = (long) exist.getDuration();
            if (duration - DateUtil.between(exist.getCurrentDate(), DateUtil.date(), DateUnit.SECOND) < ThirdPartyApiConstant.MAX_SECOND) {
                loginInfo = this.getSanDieToken(context);
            } else {
                loginInfo = exist.getCode();
            }
        }
        return XSingleResult.success(JSON.parseObject(loginInfo, GetThirdSanDieLoginInfoOutput.class));
    }

    /*---------------------------------- 私有方法 ----------------------------------*/

    /**
     * 获取三叠令牌
     *
     * @param context 上下文
     * @return {@link String}
     */
    private String getSanDieToken(XContext context) {
        JSONObject param = new JSONObject();
        param.put("username", "31b5604627e345ebb48175de8f2ee0bdem55ajAwMQ==kj");
        param.put("password", "sdSktfem55ajAwMSFAIzEw8ef354f215d44c0eafd403ab7a1ce74a");
        String result = XHttpUtils.postAsJson(ThirdPartyApiConstant.CQ_SD_LOGIN, param.toJSONString());
        context.getLogger().info("**** getThirdSanDieLoginInfo result: {}", result);
        param = JSON.parseObject(result);
        param = param.getJSONObject("data");
        if (ObjectUtil.isEmpty(param)) {
            throw new XServiceException(context, BusinessError.GetSanDieLoginInfoError);
        }
        String data = param.toJSONString();
        ThirdPartyConfigCache cache = new ThirdPartyConfigCache();
        cache.setCodeKey(ThirdPartyApiConstant.CQ_TOKEN_SAN_DIE_CACHE_KEY);
        cache.setCode(data);
        cache.setValidity(30);
        cache.setCurrentDate(DateUtil.date());
        ThirdPartyConfigCache.set(context, cache);
        return data;
    }
}
