package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadInput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadViewOutput;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.data.GetAllOuListByOuIdInput;
import pps.cloud.system.service.data.GetSysOrganizationViewOutput;
import pps.core.prediction.entity.EnergyConsumptionAnalysisEnt;
import pps.core.prediction.entity.ThirdPhotovoltaicPowerEnt;
import pps.core.prediction.entity.WellheadDailyProductionSituationEnt;
import pps.core.prediction.mapper.ThirdPhotovoltaicPowerMapper;
import pps.core.prediction.mapper.WellheadDailyProductionSituationMapper;
import pps.core.prediction.service.data.third_photovoltaic_power.GetThirdPhotovoltaicPowerInput;
import pps.core.prediction.service.data.third_photovoltaic_power.GetThirdPhotovoltaicPowerOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiPost;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 第三方光伏日发电量
 *
 * @author ZWT
 * @date 2024/03/19
 */
@XText("第三方光伏日发电量")
@XService
public class ThirdPhotovoltaicPowerService {

    /**
     * 获取光伏发电概述
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetThirdPhotovoltaicPowerOutput}>
     */
    @XText("能耗分析--能耗概览")
    @XApiPost
    public XSingleResult<GetThirdPhotovoltaicPowerOutput> getPhotovoltaicPowerOverview(XContext context, GetThirdPhotovoltaicPowerInput input) {
        GetThirdPhotovoltaicPowerOutput output = new GetThirdPhotovoltaicPowerOutput();
        List<DynamicQueryBasePowerLineOutput> lineList = this.getBasePowerLineListByOuIdList(context, this.getOrgIdsByPath(context, input.getOuId()));
        if (CollUtil.isEmpty(lineList)) {
            output.setPowerGeneration(BigDecimal.ZERO);
            output.setPowerConsumption(BigDecimal.ZERO);
            output.setAbsorptionRate(BigDecimal.ZERO);
        } else {
            List<String> lineNameList = new ArrayList<>(lineList.size());
            List<String> lineIdList = new ArrayList<>(lineList.size());
            lineList.forEach(line -> {
                lineNameList.add(line.getLineName());
                lineIdList.add(line.getId());
            });
            input.getBetweenDate();
            BigDecimal powerConsumption;
            //查井口
            List<DynamicQueryBasePowerLineWellheadViewOutput> wellheadList = this.getBaseWellheadListByLineIdList(context, lineIdList);
            if (CollUtil.isEmpty(lineList)) {
                powerConsumption = BigDecimal.ONE;
            } else {
                WellheadDailyProductionSituationMapper wellheadMapper = context.getBean(WellheadDailyProductionSituationMapper.class);
                WellheadDailyProductionSituationEnt wellheadSituationEnt = wellheadMapper.selectOne(new QueryWrapper<WellheadDailyProductionSituationEnt>()
                        .select("IFNULL( SUM( daily_electricity_consumption ), 0 ) AS daily_electricity_consumption")
                        .lambda()
                        .in(WellheadDailyProductionSituationEnt::getWellheadId, wellheadList.stream()
                                .map(DynamicQueryBasePowerLineWellheadViewOutput::getWellheadId)
                                .collect(Collectors.toSet()))
                        .between(WellheadDailyProductionSituationEnt::getDataDate, input.getStartTime(), input.getEndTime())
                );
                powerConsumption = wellheadSituationEnt.getDailyElectricityConsumption();
            }
            //查光伏发电
            ThirdPhotovoltaicPowerMapper powerMapper = context.getBean(ThirdPhotovoltaicPowerMapper.class);
            ThirdPhotovoltaicPowerEnt photovoltaicPowerEnt = powerMapper.selectOne(new QueryWrapper<ThirdPhotovoltaicPowerEnt>()
                    .select("IFNULL( SUM( photovoltaic_power ), 0 ) AS photovoltaic_power")
                    .lambda()
                    .in(ThirdPhotovoltaicPowerEnt::getStationName, lineNameList)
                    .between(ThirdPhotovoltaicPowerEnt::getCreateDate, input.getStartTime(), input.getEndTime())
            );
            output.setPowerGeneration(photovoltaicPowerEnt.getPhotovoltaicPower());
            output.setPowerConsumption(powerConsumption);
            output.setAbsorptionRate(this.getAbsorptionRate(photovoltaicPowerEnt.getPhotovoltaicPower(), powerConsumption));
        }
        return XSingleResult.success(output);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 分页功能封装查询条件
     *
     * @param lineIds   线路ID
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return {@link LambdaQueryWrapper}<{@link EnergyConsumptionAnalysisEnt}>
     */
    private LambdaQueryWrapper<EnergyConsumptionAnalysisEnt> getQueryWrapper4Page(Collection<String> lineIds, Date startTime, Date endTime) {
        return new QueryWrapper<EnergyConsumptionAnalysisEnt>()
                .select("IFNULL( SUM( power_generation ), 0 ) AS power_generation",
                        "IFNULL( SUM( power_consumption ), 0 ) AS power_consumption",
                        "line_id")
                .lambda()
                .in(EnergyConsumptionAnalysisEnt::getLineId, lineIds)
                .between(EnergyConsumptionAnalysisEnt::getDataDate, startTime, endTime)
                .groupBy(EnergyConsumptionAnalysisEnt::getLineId)
                .orderByAsc(EnergyConsumptionAnalysisEnt::getPowerGeneration);
    }

    /**
     * 获取消纳率
     * 消纳率:光伏发电/累计用电*100%
     *
     * @param powerGeneration  发电
     * @param powerConsumption 功耗
     * @return {@link BigDecimal}
     */
    private BigDecimal getAbsorptionRate(BigDecimal powerGeneration, BigDecimal powerConsumption) {
        BigDecimal absorptionRate;
        if (powerConsumption.compareTo(BigDecimal.ZERO) > 0) {
            absorptionRate = powerGeneration.divide(powerConsumption, 4, RoundingMode.HALF_UP)
                    .multiply(new BigDecimal(100));
        } else {
            absorptionRate = BigDecimal.ZERO;
        }
        return absorptionRate;
    }

    /**
     * 获取环比
     * (本期-上期) /上期*100%
     *
     * @param currentPeriod 本期
     * @param lastPeriod    上期
     * @return {@link BigDecimal}
     */
    private BigDecimal getCycleRatio(BigDecimal currentPeriod, BigDecimal lastPeriod) {
        BigDecimal cycleRatio;
        if (lastPeriod.compareTo(BigDecimal.ZERO) > 0) {
            cycleRatio = currentPeriod.subtract(lastPeriod)
                    .divide(lastPeriod, 4, RoundingMode.HALF_UP)
                    .multiply(new BigDecimal(100));
        } else {
            cycleRatio = BigDecimal.ZERO;
        }
        return cycleRatio;
    }

    /**
     * 获取所有path包含组织ID的组织
     *
     * @param context 上下文
     * @param ouId    ou id
     * @return {@link List}<{@link String}>
     */
    private List<String> getOrgIdsByPath(XContext context, String ouId) {
        SysOrganizationCloudService organizationCloudService = context.getBean(SysOrganizationCloudService.class);
        GetAllOuListByOuIdInput ouIdInput = new GetAllOuListByOuIdInput();
        ouIdInput.setOuId(ouId);
        XListResult<GetSysOrganizationViewOutput> allListByOuId = organizationCloudService.getAllListByOuId(context, ouIdInput);
        allListByOuId.throwIfFail();
        return allListByOuId.getResult().stream()
                .map(GetSysOrganizationViewOutput::getId)
                .collect(Collectors.toList());
    }

    /**
     * 查询组织列表下所有线路
     *
     * @param context  上下文
     * @param ouIdList ou-id列表
     * @return {@link List}<{@link DynamicQueryBasePowerLineOutput}>
     */
    private List<DynamicQueryBasePowerLineOutput> getBasePowerLineListByOuIdList(XContext context, List<String> ouIdList) {
        IBasePowerLineCloudService basePowerLineCloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineOutput> lineList = basePowerLineCloudService.getBasePowerLineListByOuIdList(context, DynamicQueryBasePowerLineInput.builder()
                .ouIdList(ouIdList)
                .build());
        lineList.throwIfFail();
        return lineList.getResult();
    }

    /**
     * 查询线路列表下所有井口
     *
     * @param context    上下文
     * @param lineIdList 行id列表
     * @return {@link List}<{@link DynamicQueryBasePowerLineWellheadViewOutput}>
     */
    private List<DynamicQueryBasePowerLineWellheadViewOutput> getBaseWellheadListByLineIdList(XContext context, List<String> lineIdList) {
        IBasePowerLineCloudService basePowerLineCloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineWellheadViewOutput> wellheadList = basePowerLineCloudService.getPowerLineWellheadList(context, DynamicQueryBasePowerLineWellheadInput.builder()
                .lineIds(lineIdList)
                .build());
        wellheadList.throwIfFail();
        return wellheadList.getResult();
    }
}
