package pps.core.prediction.service.data.energy_consumption_analysis;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import pps.core.prediction.enums.BusinessError;
import xstartup.annotation.XText;
import xstartup.base.exception.XServiceException;

import java.util.Date;

/**
 * 能耗分析表(测试用)
 *
 * @author ZWT
 * @date 2023/11/23
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
public class QueryEnergyConsumptionAnalysisInput {
    /**
     * 组织机构ID
     */
    @NotBlank(message = "缺少组织机构ID")
    @XText("组织机构ID")
    private String ouId;

    /**
     * 日期类型(1_近7天,2_近30天,3_自定义)
     */
    @NotNull(message = "缺少日期类型")
    @XText("日期类型(1_近7天,2_近30天,3_自定义)")
    private Integer dateType;

    /**
     * 开始时间
     */
    @XText("开始时间")
    private Date startTime;

    /**
     * 结束时间
     */
    @XText("结束时间")
    private Date endTime;

    /**
     * 初始化开始结束日期
     * 近7天:昨天开始前七天;近30天:昨天开始前30天;上期:所选周期往前推一个周期
     */
    public void getBetweenDate() {
        switch (this.dateType) {
            case 1:
                this.endTime = DateUtil.beginOfDay(DateUtil.yesterday());
                this.startTime = DateUtil.offsetDay(endTime, -6);
                break;
            case 2:
                this.endTime = DateUtil.beginOfDay(DateUtil.yesterday());
                this.startTime = DateUtil.offsetDay(endTime, -29);
                break;
            case 3:
                if (ObjectUtil.isNull(this.endTime) || ObjectUtil.isNull(this.startTime)) {
                    throw new XServiceException(BusinessError.TheStartEndDateIsMissing);
                }
                break;
            default:
                throw new XServiceException(BusinessError.DateTypeError);
        }
    }
}
